# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""Filter search results based on selected facets."""

import json

from itertools import groupby
from operator import itemgetter

from flask import request

from invenio_search.registry import facets

from invenio_query_parser.ast import (
    AndOp, DoubleQuotedValue, Keyword, KeywordOp, NotOp, OrOp
)


def facet_formatter(key, val):
    """Format the operation that is part or the bool expressions."""
    return KeywordOp(Keyword(key), DoubleQuotedValue(val))


def get_groupped_facets(filter_data):
    """Group facets based on keyword and operator.

    :param filter_data: the request filter parameter
    :return: a dictionary with grouped facets
    """
    # Group filter data by operator and then by facet key.
    sortkeytype = itemgetter(0)
    sortfacet = itemgetter(1)
    data = sorted(filter_data, key=sortkeytype)
    out = {}
    for t, vs in groupby(data, key=sortkeytype):
        out[t] = {}
        for v, k in groupby(sorted(vs, key=sortfacet), key=sortfacet):
            out[t][v] = map(lambda i: i[2], k)
    return out


def format_facet_tree_nodes(query, filter_data, facets,
                            formatter=facet_formatter):
    """Add extra nodes to the AST.

    First get the facet filter expression from the request values.
    Group the facets and create the new AST nodes to be added.
    Combine the new nodes with the initial query in an AndOp.

    :param query: the original AST
    :param filter_data: the grouped facets dictionary
    :param facets: a list with the configured facet keys
    :return: the facet filter AST
    """
    # Intersect and diff records with selected facets.
    def union_facet_values(key, values):
        return reduce(OrOp, [formatter(key, value) for value in values])

    if '+' in filter_data:
        values = filter_data['+']
        for key in facets:
            if key in values:
                query = AndOp(query, union_facet_values(key, values[key]))

    if '-' in filter_data:
        values = filter_data['-']
        for key in facets:
            if key in values:
                query = AndOp(query, NotOp(
                    union_facet_values(key, values[key]))
                )

    return query


def apply(query, user_info=None, collection=None, **kwargs):
    """Enhance the query AST with the facet filters."""
    if 'filter' in request.values:
        if 'filter' in request.values:
            filter_data = get_groupped_facets(json.loads(
                request.values.get('filter', '[]')
            ))
            query = format_facet_tree_nodes(query, filter_data, facets)
    return query



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2013, 2014 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""Unit tests for search views."""

from intbitset import intbitset
from flask import url_for, current_app
from invenio.testsuite import InvenioTestCase, make_test_suite, \
    run_test_suite


class SearchViewTest(InvenioTestCase):
    """ Test search view functions. """

    def test_home_collection_page_availability(self):
        response = self.client.get(url_for('collections.index'))
        self.assert200(response)

        response = self.client.get(url_for(
            'collections.collection', name=current_app.config['CFG_SITE_NAME']))
        self.assert200(response)

    def test_search_page_availability(self):
        response = self.client.get(url_for('search.search'))
        self.assert200(response)


TEST_SUITE = make_test_suite(SearchViewTest)

if __name__ == "__main__":
    run_test_suite(TEST_SUITE)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2014, 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""Search engine API."""

from __future__ import unicode_literals

import pypeg2

from flask_login import current_user

from invenio.base.helpers import unicodifier

from werkzeug.utils import cached_property

from .utils import parser, query_enhancers, query_walkers, search_walkers
from .walkers.match_unit import MatchUnit
from .walkers.terms import Terms


class Query(object):

    """Search engine implemetation.

    .. versionadded:: 2.1
       New search and match API.
    """

    def __init__(self, query):
        """Initialize with search query."""
        self._query = unicodifier(query)

    @cached_property
    def query(self):
        """Parse query string using given grammar."""
        tree = pypeg2.parse(self._query, parser(), whitespace="")
        for walker in query_walkers():
            tree = tree.accept(walker)
        return tree

    def search(self, user_info=None, collection=None, **kwargs):
        """Search records."""
        user_info = user_info or current_user
        # Enhance query first
        query = self.query
        for enhancer in query_enhancers():
            query = enhancer(query, user_info=user_info,
                             collection=collection)

        for walker in search_walkers():
            query = query.accept(walker)
        return Results(query)


    def match(self, record, user_info=None):
        """Return True if record match the query."""
        return self.query.accept(MatchUnit(record))

    def terms(self, keywords=None):
        """Return list of terms for given keywords in query pattern."""
        return self.query.accept(Terms(keywords=keywords))


class Results(object):

    def __init__(self, query, **kwargs):
        self.body = {
            'from': 0,
            'size': 10,
            'query': query,
        }
        self.body.update(kwargs)

        self._results = None

    @property
    def recids(self):
        # FIXME add warnings
        from intbitset import intbitset
        from invenio.ext.es import es
        results = es.search(
            index='records',
            doc_type='record',
            body={
                'size': 9999999,
                'fields': ['control_number'],
                'query': self.body.get("query")
            }
        )
        return intbitset([int(r['_id']) for r in results['hits']['hits']])

    def _search(self):
        from invenio.ext.es import es

        if self._results is None:
            self._results = es.search(
                index='records',
                doc_type='record',
                body=self.body,
            )
        return self._results

    def records(self):
        from invenio_records.api import Record
        return [Record(r['_source']) for r in self._search()['hits']['hits']]

    def __len__(self):
        return self._search()['hits']['total']



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""Generate filter for retricted collections."""

from invenio.base.globals import cfg

from invenio_query_parser.ast import (
    AndOp, DoubleQuotedValue, Keyword, KeywordOp, NotOp, OrOp
)


def collection_formatter(value):
    """Format collection filter."""
    return KeywordOp(Keyword("collection"), DoubleQuotedValue(value))


def create_collection_query(restricted_cols, permitted_restricted_cols,
                            current_col, policy,
                            formatter=collection_formatter):
    """Create the new AST nodes that should be added to the search query.

    **Set definitions:**

    ``cc``
        current collection

    ``pr``
        permitted-restricted collections

    ``r``
        all restrincted collections

    ``r'``
        restricted collections for current user (``r' = r - pr``)

    **Policy ANY:**

        (cc AND NOT r') OR (cc and pr) -> cc AND (NOT r' OR rp)

    **Policy ALL:**

        cc AND NOT r'

    :param restricted_cols: all restricted collections
    :param permitted_restricted_cols: restricted collections that user can
                                      access
    :param current_col: name of the current collection
    :param policy: policy applied ('ANY' or enything else)
    :param format_val: a function used to format value of the boolean
                       operation
    """
    current_col_kw = formatter(current_col)
    result_tree = current_col_kw

    def union_terms(term_list):
        return reduce(OrOp, [formatter(k) for k in term_list])

    not_permitted_cols = (
        set(restricted_cols) - set(permitted_restricted_cols)
    )

    # only if not permitted collection exists
    if not_permitted_cols:
        result_tree = NotOp(union_terms(not_permitted_cols))

        if policy == 'ANY' and permitted_restricted_cols:
            # User needs to have access to at least one collection that
            # restricts the records. We need this to be able to remove records
            # that are both in a public and restricted collection.

            result_tree = OrOp(
                result_tree,
                union_terms(permitted_restricted_cols)
            )

        # intersect current collection with restrictions
        result_tree = AndOp(current_col_kw, result_tree)

    return result_tree


def apply(query, user_info=None, collection=None):
    """Enhance the query restricting not permitted collections.

    Get the permitted restricted collection for the current user from the
    user_info object and all the restriced collections from the
    restricted_collection_cache.
    """
    from invenio_collections.cache import restricted_collection_cache

    policy = cfg['CFG_WEBSEARCH_VIEWRESTRCOLL_POLICY'].strip().upper()
    restricted_cols = restricted_collection_cache.cache
    permitted_restricted_cols = user_info.get(
        'precached_permitted_restricted_collections', [])
    current_col = collection or cfg['CFG_SITE_NAME']
    result_tree = create_collection_query(restricted_cols,
                                          permitted_restricted_cols,
                                          current_col, policy)
    return AndOp(query, result_tree)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2013, 2014, 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""Define custom action handlers."""

import os

from flask import flash, g, current_app
from logging import Formatter, getLogger, FileHandler
from six import iteritems
from werkzeug.local import LocalProxy


def get_logger():
    """Get search logger."""
    logger = getattr(g, 'search_logger', None)
    if logger is None:
        handler = FileHandler(
            os.path.join(current_app.config['CFG_LOGDIR'], 'search.log'),
            delay=True
        )
        logger = getLogger('invenio.search')
        formatter = Formatter('{asctime}#{action}#{p}#{f}#{colls}#{total}',
                              datefmt='%Y%m%d%H%M%S', style='{')

        handler.setFormatter(formatter)
        logger.addHandler(handler)
        g.search_logger = logger
    return logger

logger = LocalProxy(get_logger)


def websearch_before_browse_handler(collection, **kwargs):
    """Flash message before browsing handler is called."""
    # keys = ['p', 'p1', 'p2', 'p3', 'f', 'f1', 'f2', 'f3', 'rm', 'cc', 'ln',
    #         'jrec', 'rg', 'aas', 'action']
    # kwargs = dict(filter(lambda (k, v): k in keys, iteritems(kwargs)))
    # if kwargs.get('action', '') == 'browse':
    # if msg and len(msg) > 0:
    #     flash(_("Did you mean to browse in %{x_index_name} index?",
    #             url), 'websearch-after-search-form')


def after_search(app, **kwargs):
    """Log user query after search."""
    from .models import UserQuery
    UserQuery.log()
    logger.info(extra=kwargs)


def after_insert_user_query():
    """Flash message after user query is logged."""
    #  of = request.values.get('of', 'hb')
    #  if of.startswith("h") and (em == '' or EM_REPOSITORY["alert"] in em):
    #      if not of in ['hcs', 'hcs2']:
    #          # display alert/RSS teaser for non-summary formats:
    #          display_email_alert_part = True
    #          if current_user:
    #              if current_user['email'] == 'guest':
    #                  if CFG_ACCESS_CONTROL_LEVEL_ACCOUNTS > 4:
    #                      display_email_alert_part = False
    #              else:
    #                  if not current_user['precached_usealerts']:
    #                      display_email_alert_part = False
    #          from flask import flash
    #          flash(websearch_templates.tmpl_alert_rss_teaser_box_for_query(
    #               id_query,
    #               ln=ln,
    #               display_email_alert_part=display_email_alert_part),
    #               'search-results-after')
    pass



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2011, 2012, 2013 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""Unit tests for the search engine summarizer."""

# Note: citation summary tests were moved to BibRank as part of the
# self-cite commit 1fcbed0ec34a9c31f8a727e21890c529d8222256.  Keeping
# this file here with empty test case set in order to overwrite any
# previously installed file.  Also, keeping TEST_SUITE empty so that
# `inveniocfg --run-unit-tests' would not complain.

from invenio.testsuite import make_test_suite, run_test_suite

TEST_SUITE = make_test_suite()

if __name__ == "__main__":
    run_test_suite(TEST_SUITE)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2014, 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""Search bundles."""

from __future__ import unicode_literals

from invenio.base.bundles import invenio as _i, jquery as _j
from invenio.ext.assets import Bundle, RequireJSFilter

js = Bundle(
    'js/search/init.js',
    filters=RequireJSFilter(exclude=[_j, _i]),
    output="search.js",
    weight=50,
    bower={
        "flight": "latest",
    }
)

styles = Bundle(
    'css/search/collection.css',
    'css/search/search.css',
    'css/search/searchbar.css',
    filters="cleancss",
    output="search.css",
    weight=60
)

adminjs = Bundle(
    'js/admin/search/init.js',
    filters=RequireJSFilter(exclude=[_j, _i]),
    output="admin/search.js",
    weight=50,
    bower={
        "jquery-ui": "~1.11",
    }
)



# -*- coding: utf-8 -*-

# This file is part of Invenio.
# Copyright (C) 2014, 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""WebSearch services unit tests."""

from invenio.testsuite import InvenioTestCase, make_test_suite, run_test_suite


class WebSearchServicesJournalHintService(InvenioTestCase):

    """Check JournalHintService plugin."""

    def setUp(self):
        """Load plugin."""
        from invenio_search.searchext.services.JournalHintService import \
            JournalHintService
        self.plugin = JournalHintService()

    def test_seems_a_journal_reference(self):
        """Check function JournalHintService.seems_a_journal_reference."""
        test_pairs = [
            # Empty strings
            ("", False),
            (" ", False),
            ("\t", False),
            ("\t \t", False),
            # Invenio and Inspire queries
            ("author:Tom", False),
            ("find a John", False),
            ("find date after 2001", False),
            ("year:2001 OR year:2002", False),
            # Invalid journals with bad format
            ("Monkey", False),
            ("Monkey monkey", False),
            ("Monkey monkey (1)", False),
            ("Monkey monkey 1234", False),
            # Invalid journals with good format
            ("Monkey monkey, 1234", True),
            ("Monkey (2000)", True),
            # Invalid journals with good format, utf8
            (u"Capellà Pere (2000)".encode("utf8"), True),
            (u"País Valencià, 1234".encode("utf8"), True),
            # Valid journal with good format
            ("JHEP 9906:028 (1999)", True),
            ("Phys. Rev. Lett. 83 (1999) 3605", True),
            ("Nucl.Phys.,B75,(1974),461", True),
            ("  Nucl.  Phys.   B75   (1974)  461   ", True),
            ("D.S. Salopek, J.R.Bond and J.M.Bardeen,Phys.Rev.D40(1989)1753.",
             True),
        ]

        for test_input, expected in test_pairs:
            test_output = self.plugin.seems_a_journal_reference(test_input)
            self.assertEqual(
                test_output,
                expected,
                "JournalHintService.seems_a_journal_reference('%s') "
                "returned '%s' instead of '%s'"
                % (repr(test_input), repr(test_output), repr(expected))
            )


TEST_SUITE = make_test_suite()

if __name__ == "__main__":
    run_test_suite(TEST_SUITE)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2012, 2013, 2014, 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""
WebSearch Flask Blueprint.

Template hierarchy.
-------------------

- ``searchbar_frame_base.html``
    - ``searchbar_frame.html``
        - ``collection_base.html``
            - ``collection.html`` used by ``/collection/<collection>``
                - ``index_base.html``
                    - ``index.html`` used by ``/``
        - ``search_base.html``
            - ``search.html``
        - ``browse_base.html``
            - ``browse.html`` used by ``/browse``
    - ``results_base.html``
        - ``results.html``
- ``helpers_base.html`` macros
    - ``helpers.html``

"""

import datetime
import time


from flask import (Blueprint, abort, current_app, flash, g, jsonify,
                   make_response, redirect, render_template, request,
                   url_for)
from flask_breadcrumbs import (default_breadcrumb_root,
                               register_breadcrumb)
from flask_login import current_user

from invenio.base.decorators import wash_arguments
from invenio.base.globals import cfg
from invenio.base.i18n import _
from invenio_collections.decorators import check_collection
from invenio_formatter import (
    format_records, get_output_format_content_type,
    response_formated_records
)
from invenio.utils.pagination import Pagination

from werkzeug.http import http_date
from werkzeug.local import LocalProxy

from ..api import Query
from ..forms import EasySearchForm

blueprint = Blueprint('search', __name__, url_prefix="",
                      template_folder='../templates',
                      static_url_path='',  # static url path has to be empty
                                           # if url_prefix is empty
                      static_folder='../static')

default_breadcrumb_root(blueprint, '.')


def _collection_of():
    """Get output format from user settings."""
    of = current_user['settings'].get('of')
    if of is not None and of != '':
        return of
    return g.collection.formatoptions[0]['code']

collection_of = LocalProxy(_collection_of)


def _default_rg():
    """Get number of records per page from user settings."""
    rg = cfg['CFG_WEBSEARCH_DEF_RECORDS_IN_GROUPS']
    if 'rg' not in request.values and current_user.get('rg'):
        rg = current_user.get('rg')
    return int(rg) or 1

default_rg = LocalProxy(_default_rg)


"""Collection output format."""


def min_length(length, code=406):
    """TODO."""
    def checker(value):
        if len(value) < 3:
            abort(code)
        return value
    return checker


def crumb_builder(url):
    """TODO."""
    def _crumb_builder(collection):
        qargs = request.args.to_dict()
        qargs['cc'] = collection.name
        return dict(text=collection.name_ln, url=url_for(url, **qargs))
    return _crumb_builder


def collection_breadcrumbs(collection, endpoint=None):
    """TODO."""
    b = []
    if endpoint is None:
        endpoint = request.endpoint
    if collection.id > 1:
        qargs = request.values.to_dict()
        k = 'cc' if 'cc' in qargs else 'c'
        del qargs[k]
        b = [(_('Home'), endpoint, qargs)] + collection.breadcrumbs(
            builder=crumb_builder(endpoint), ln=g.ln)[1:]
    return b


@blueprint.route('/browse', methods=['GET', 'POST'])
@register_breadcrumb(blueprint, '.browse', _('Browse results'))
@wash_arguments({'p': (unicode, ''),
                 'f': (unicode, None),
                 'of': (unicode, 'hb'),
                 'so': (unicode, None),
                 'rm': (unicode, None),
                 'rg': (int, default_rg),
                 'jrec': (int, 1)})
@check_collection(default_collection=True)
def browse(collection, p, f, of, so, rm, rg, jrec):
    """Render browse page."""
    return 'Not supported.'


@blueprint.route('/rss', methods=['GET'])
# FIXME caching issue of response object
@wash_arguments({'p': (unicode, ''),
                 'jrec': (int, 1),
                 'so': (unicode, None),
                 'rm': (unicode, None),
                 'rg': (int, default_rg)})
@check_collection(default_collection=True)
def rss(collection, p, jrec, so, rm, rg):
    """Render RSS feed."""
    response = Query(p).search(collection=collection.name)
    response.body.update({
        'size': rg,
        'from': jrec-1,
    })

    return response_formated_records(
        response.records(), 'xr',
        records=len(response),
        collection=collection,
        rg=rg,
    )


@blueprint.route('/search', methods=['GET', 'POST'])
@register_breadcrumb(blueprint, '.browse', _('Search results'))
@wash_arguments({'p': (unicode, ''),
                 'of': (unicode, collection_of),
                 'ot': (unicode, None),
                 'so': (unicode, None),
                 'sf': (unicode, None),
                 'sp': (unicode, None),
                 'rm': (unicode, None),
                 'rg': (int, default_rg),
                 'jrec': (int, 1)})
@check_collection(default_collection=True)
def search(collection, p, of, ot, so, sf, sp, rm, rg, jrec):
    """Render search page."""
    if 'action_browse' in request.args \
            or request.args.get('action', '') == 'browse':
        return browse()

    if 'c' in request.args and len(request.args) == 1 \
            and len(request.args.getlist('c')) == 1:
        return redirect(url_for('.collection', name=request.args.get('c')))

    if 'f' in request.args:
        args = request.args.copy()
        args['p'] = "{0}:{1}".format(args['f'], args['p'])
        del args['f']
        return redirect(url_for('.search', **args))

    # fix for queries like `/search?p=+ellis`
    p = p.strip().encode('utf-8')

    collection_breadcrumbs(collection)

    response = Query(p).search(collection=collection.name)
    response.body.update({
        'size': int(rg),
        'from': jrec-1,
        'aggs': {
            "collection": {"terms": {"field": "_collections"}},
            "author": {"terms": {"field": "authors.raw"}},
        },
    })

    # FIXME refactor to separate search hook
    filtered_facets = ''
    from invenio_search.walkers.elasticsearch import ElasticSearchDSL
    if 'post_filter' in request.values:
        parsed_post_filter = Query(request.values.get('post_filter'))
        post_filter = parsed_post_filter.query.accept(
            ElasticSearchDSL()
        )
        response.body['post_filter'] = post_filter
        # extracting the facet filtering
        from invenio_search.walkers.facets import FacetsVisitor
        filtered_facets = parsed_post_filter.query.accept(
            FacetsVisitor()
        )
        # sets cannot be converted to json. use facetsVisitor to convert them to
        # lists
        filtered_facets = FacetsVisitor.jsonable(filtered_facets)

    if len(response) and jrec > len(response):
        args = request.args.copy()
        args['jrec'] = 1
        return redirect(url_for('.search', **args))

    pagination = Pagination((jrec-1) // rg + 1, rg, len(response))

    ctx = dict(
        facets={},  # facets.get_facets_config(collection, qid),
        filtered_facets=filtered_facets,
        response=response,
        rg=rg,
        create_nearest_terms_box=lambda: _("Try to modify the query."),
        easy_search_form=EasySearchForm(csrf_enabled=False),
        ot=ot,
        pagination=pagination,
        collection=collection,
    )

    # TODO add search services
    # TODO add external collection search

    return response_formated_records(response.records(), of, **ctx)


@blueprint.route('/facet/<name>/<qid>', methods=['GET', 'POST'])
def facet(name, qid):
    """
    Create list of fields specified facet.

    :param name: facet identifier
    :param qid: query identifier

    :return: jsonified facet list sorted by number of records
    """
    return jsonify(facet={})


@blueprint.route('/list/<any(exactauthor, keyword, affiliation, reportnumber, '
                 'collaboration):field>',
                 methods=['GET', 'POST'])
@wash_arguments({'q': (min_length(3), '')})
def autocomplete(field, q):
    """Autocomplete data from indexes.

    It uses POSTed arguments with name `q` that has to be longer than 3
    characters in order to returns any results.

    :param field: index name
    :param q: query string for index term

    :return: list of values matching query.
    """
    return jsonify(results={})


@blueprint.route('/search/dispatch', methods=['GET', 'POST'])
def dispatch():
    """Redirect request to appropriate methods from search page."""
    action = request.values.get('action')
    if action not in ['export']:
        abort(406)

    if action == 'export':
        return redirect(url_for('record.export',
                                **request.values.to_dict(flat=False)))

    flash("Not implemented action " + action, 'error')
    return redirect(request.referrer)


@blueprint.route('/opensearchdescription')
def opensearchdescription():
    """Render OpenSearch description file."""
    response = make_response(render_template(
        'search/opensearchdescription.xml'))
    response.headers['Content-Type'] = 'application/opensearchdescription+xml'
    return response



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2012, 2013, 2014, 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""Facet utility functions."""

from flask import g, url_for, request
from flask_login import current_user
from intbitset import intbitset
from itertools import groupby
from operator import itemgetter
from six import iteritems

from invenio.base.globals import cfg
from invenio_collections.models import Collection

from .cache import (
    get_search_results_cache_key_from_qid,
    search_results_cache,
)
from .models import Field
from .utils import (
    get_most_popular_field_values,
    get_records_that_can_be_displayed,
)


def get_current_user_records_that_can_be_displayed(qid):
    """Return records that current user can display.

    :param qid: query identifier

    :return: records in intbitset
    """
    CFG_WEBSEARCH_SEARCH_CACHE_TIMEOUT = cfg.get(
        'CFG_WEBSEARCH_SEARCH_CACHE_TIMEOUT')

    @search_results_cache.memoize(timeout=CFG_WEBSEARCH_SEARCH_CACHE_TIMEOUT)
    def get_records_for_user(qid, uid):
        key = get_search_results_cache_key_from_qid(qid)
        data = search_results_cache.get(key)
        if data is None:
            return intbitset([])
        cc = search_results_cache.get(key + '::cc')
        return get_records_that_can_be_displayed(
            current_user.get('precached_permitted_restricted_collections', []),
            intbitset().fastload(data), cc)
    # Simplifies API
    return get_records_for_user(qid, current_user.get_id())


def faceted_results_filter(recids, filter_data, facets):
    """Return records that match selected filter data.

    :param recids: found records
    :param filter_date: selected facet filters
    :param facet_config: facet configuration

    :return: filtered records
    """
    # Group filter data by operator and then by facet key.
    sortkeytype = itemgetter(0)
    sortfacet = itemgetter(1)
    data = sorted(filter_data, key=sortkeytype)
    out = {}
    for t, vs in groupby(data, key=sortkeytype):
        out[t] = {}
        for v, k in groupby(sorted(vs, key=sortfacet), key=sortfacet):
            out[t][v] = map(lambda i: i[2], k)

    filter_data = out

    # Intersect and diff records with selected facets.
    output = recids

    if '+' in filter_data:
        values = filter_data['+']
        for key, facet in iteritems(facets):
            if key in values:
                output.intersection_update(facet.get_facet_recids(values[key]))

    if '-' in filter_data:
        values = filter_data['-']
        for key, facet in iteritems(facets):
            if key in values:
                output.difference_update(facet.get_facet_recids(values[key]))

    return output


class FacetBuilder(object):

    """Facet builder helper class."""

    def __init__(self, name):
        """Initialize facet builder."""
        self.name = name

    def get_title(self, **kwargs):
        """Return facet title."""
        return g._('Any ' + self.name.capitalize())

    def get_url(self, qid=None):
        """Return facet data url."""
        return url_for('.facet', name=self.name, qid=qid)

    def get_conf(self, **kwargs):
        """Return facet configuration."""
        return dict(title=self.get_title(**kwargs),
                    url=self.get_url(kwargs.get('qid')),
                    facet=self.name)

    def get_recids_intbitset(self, qid):
        """Return record ids as intbitset."""
        try:
            return get_current_user_records_that_can_be_displayed(qid)
        except:
            return intbitset([])

    def get_recids(self, qid):
        """Return record ids as list."""
        return self.get_recids_intbitset(qid).tolist()

    def get_facets_for_query(self, qid, limit=20, parent=None):
        """Return facet data."""
        return get_most_popular_field_values(
            self.get_recids(qid), Field.get_field_tags(self.name)
        )[0:limit]

    def get_value_recids(self, value):
        """Return record ids in intbitset for given field value."""
        from .searchext.engines.native import search_unit
        if isinstance(value, unicode):
            value = value.encode('utf8')
        return search_unit(p=value, f=self.name, m='e')

    def get_facet_recids(self, values):
        """Return record ids in intbitset for all field values."""
        return reduce(lambda x, y: x.union(y),
                      [self.get_value_recids(v) for v in values],
                      intbitset())


class CollectionFacetBuilder(FacetBuilder):

    """Custom implementation of collection facet builder."""

    def get_title(self, **kwargs):
        """Return title for collection facet."""
        collection = kwargs.get('collection')
        if collection is not None and collection.id > 1:
            return collection.name_ln
        return super(CollectionFacetBuilder, self).get_title(**kwargs)

    def get_facets_for_query(self, qid, limit=20, parent=None):
        """Return record ids as intbitset."""
        pass



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2011, 2012, 2013, 2014, 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""Database models for search engine."""

import datetime

from flask import g, request

from flask_login import current_user

from invenio.base.globals import cfg
from invenio.ext.sqlalchemy import db
from invenio_accounts.models import User

from sqlalchemy.schema import Index


class Field(db.Model):

    """Represent a Field record."""

    def __repr__(self):
        """Get repr."""
        return "%s(%s)" % (self.__class__.__name__, self.id)

    __tablename__ = 'field'
    id = db.Column(db.MediumInteger(9, unsigned=True),
                   primary_key=True)
    name = db.Column(db.String(255), nullable=False)
    code = db.Column(db.String(255), unique=True,
                     nullable=False)

    @property
    def name_ln(self):
        """Get name ln."""
        from .cache import get_field_i18nname
        return get_field_i18nname(self.name,
                                  getattr(g, 'ln', cfg['CFG_SITE_LANG']))
        # try:
        #    return db.object_session(self).query(Fieldname).\
        #        with_parent(self).filter(db.and_(Fieldname.ln==g.ln,
        #            Fieldname.type=='ln')).first().value
        # except Exception:
        #    return self.name

    @classmethod
    def get_field_name(cls, code):
        """Return field name for given code."""
        return cls.query.filter_by(code=code).value(cls.name)

    @classmethod
    def get_field_tags(cls, code, tagtype='marc'):
        """Yield tag values for given field code."""
        column = Tag.value if tagtype == 'marc' else Tag.recjson_value
        tags = cls.query.join(cls.tags).join(FieldTag.tag).filter(
            cls.code == code
        ).values(column)
        for tag in tags:
            for value in tag[0].split(','):
                yield value.strip()


class Fieldvalue(db.Model):

    """Represent a Fieldvalue record."""

    def __init__(self):
        """Init."""
        pass
    __tablename__ = 'fieldvalue'
    id = db.Column(db.MediumInteger(9, unsigned=True),
                   primary_key=True,
                   autoincrement=True)
    name = db.Column(db.String(255), nullable=False)
    value = db.Column(db.Text, nullable=False)


class Fieldname(db.Model):

    """Represent a Fieldname record."""

    __tablename__ = 'fieldname'
    id_field = db.Column(db.MediumInteger(9, unsigned=True),
                         db.ForeignKey(Field.id), primary_key=True)
    ln = db.Column(db.Char(5), primary_key=True, server_default='')
    type = db.Column(db.Char(3), primary_key=True, server_default='sn')
    value = db.Column(db.String(255), nullable=False)
    field = db.relationship(Field, backref='names')


class Tag(db.Model):

    """Represent a Tag record."""

    __tablename__ = 'tag'
    id = db.Column(db.MediumInteger(9, unsigned=True), primary_key=True)
    name = db.Column(db.String(255), nullable=False)
    value = db.Column(db.Char(6), nullable=False, server_default='')
    recjson_value = db.Column(db.Text, nullable=False)

    def __init__(self, tup=None, *args, **kwargs):
        """Init."""
        if tup is not None and isinstance(tup, tuple):
            self.name, self.value = tup
            super(Tag, self).__init__(*args, **kwargs)
        else:
            if tup is None:
                super(Tag, self).__init__(*args, **kwargs)
            else:
                super(Tag, self).__init__(tup, *args, **kwargs)

    @property
    def as_tag(self):
        """Return tupple with name and value."""
        return self.name, self.value


class FieldTag(db.Model):

    """Represent a FieldTag record."""

    __tablename__ = 'field_tag'
    id_field = db.Column(db.MediumInteger(9, unsigned=True),
                         db.ForeignKey('field.id'), nullable=False,
                         primary_key=True)
    id_tag = db.Column(db.MediumInteger(9, unsigned=True),
                       db.ForeignKey('tag.id'), nullable=False,
                       primary_key=True)
    score = db.Column(db.TinyInteger(4, unsigned=True), nullable=False,
                      server_default='0')
    tag = db.relationship(Tag, backref='fields', order_by=score)
    field = db.relationship(Field, backref='tags', order_by=score)

    def __init__(self, score=None, tup=None, *args, **kwargs):
        """Init."""
        if score is not None:
            self.score = score
        if tup is not None:
            self.tag = Tag(tup)
        super(FieldTag, self).__init__(*args, **kwargs)

    @property
    def as_tag(self):
        """Return Tag record directly."""
        return self.tag


class WebQuery(db.Model):

    """Represent a WebQuery record."""

    __tablename__ = 'query'
    id = db.Column(db.Integer(15, unsigned=True), primary_key=True,
                   autoincrement=True)
    type = db.Column(db.Char(1), nullable=False, server_default='r')
    urlargs = db.Column(
        db.Text().with_variant(db.Text(100), 'mysql'),
        nullable=False)


Index('ix_query_urlargs', WebQuery.urlargs, mysql_length=100)


class UserQuery(db.Model):

    """Represent a UserQuery record."""

    __tablename__ = 'user_query'
    id_user = db.Column(db.Integer(15, unsigned=True),
                        db.ForeignKey(User.id), primary_key=True,
                        server_default='0')
    id_query = db.Column(db.Integer(15, unsigned=True),
                         db.ForeignKey(WebQuery.id), primary_key=True,
                         index=True, server_default='0')
    hostname = db.Column(db.String(50), nullable=True,
                         server_default='unknown host')
    date = db.Column(db.DateTime, nullable=True,
                     default=datetime.datetime.now)

    webquery = db.relationship(WebQuery, backref='executions')

    @classmethod
    def log(cls, urlargs=None, id_user=None):
        """Log."""
        id_user = id_user if not None else current_user.get_id()
        urlargs = urlargs or request.query_string
        if id_user < 0:
            return
        webquery = WebQuery.query.filter_by(urlargs=urlargs).first()
        if webquery is None:
            webquery = WebQuery(urlargs=urlargs)
        db.session.add(cls(id_user=id_user, hostname=request.host,
                           webquery=webquery))
        db.session.commit()


__all__ = (
    'Field',
    'Fieldvalue',
    'Fieldname',
    'Tag',
    'FieldTag',
    'WebQuery',
    'UserQuery',
)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2011, 2014, 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""WebMessage Forms."""

from flask import url_for
from six import iteritems

from invenio.base.i18n import _
from invenio.modules.knowledge.api import get_kb_mappings
from invenio.utils.forms import AutocompleteField, InvenioBaseForm, \
    RowWidget

from werkzeug.local import LocalProxy

from wtforms import Form as WTFormDefault, FormField, SelectField, \
    SelectMultipleField, StringField


class JournalForm(WTFormDefault):

    """Journal Form."""

    name = SelectField(
        label='',
        choices=LocalProxy(
            lambda:
            [('', _('Any journal'))] +
            [(kb['key'], kb['value']) for kb in get_kb_mappings('EJOURNALS')]),
        coerce=unicode,
    )
    vol = StringField(_('Vol'))
    page = StringField(_('Pg'))


class EasySearchForm(InvenioBaseForm):

    """Defines form for easy seach popup."""

    author = AutocompleteField(_('Author'), data_provide="typeahead-url",
                               data_source=lambda:
                               url_for('search.autocomplete',
                                       field='exactauthor'))
    title = StringField(_('Title'))
    rn = AutocompleteField(_('Report number'), data_provide="typeahead-url",
                           data_source=lambda: url_for('search.autocomplete',
                                                       field='reportnumber'))
    aff = AutocompleteField(_('Affiliation'), data_provide="typeahead-url",
                            data_source=lambda: url_for('search.autocomplete',
                                                        field='affiliation'))
    cn = AutocompleteField(_('Collaboration'), data_provide="typeahead-url",
                           data_source=lambda: url_for('search.autocomplete',
                                                       field='collaboration'))
    k = AutocompleteField(_('Keywords'), data_provide="typeahead-url",
                          data_source=lambda: url_for('search.autocomplete',
                                                      field='keyword'))
    journal = FormField(JournalForm, widget=RowWidget(
        classes={0: 'col-xs-6', 1: 'col-xs-3', 2: 'col-xs-3'}))


class GetCollections(object):

    """Get all collections."""

    def __iter__(self):
        """Get all the collections."""
        from invenio_collections.models import Collection
        collections = Collection.query.all()

        for coll in collections:
            if not coll.is_restricted:
                yield (coll.name, coll.name_ln)


class GetOutputFormats(object):

    """Collection output formats."""

    def __iter__(self):
        """Get all the output formats."""
        from invenio_formatter import registry

        yield ('', _('Default'))
        for code, format_ in iteritems(registry.output_formats):
            yield (code, format_['name'])


class WebSearchUserSettingsForm(InvenioBaseForm):

    """User settings for search."""

    rg = SelectField(_('Results per page'),
                     choices=[('10', '10'), ('25', '25'), ('50', '50'),
                              ('100', '100')])
    websearch_hotkeys = SelectField(_('Hotkeys'), choices=[('0', _('Disable')),
                                                           ('1', _('Enable'))])

    c = SelectMultipleField(_('Collections'), choices=GetCollections())
    of = SelectField(_('Personal output format'), choices=GetOutputFormats())



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""Upgrade Search models."""

from invenio.ext.sqlalchemy import db
from invenio.legacy.dbquery import run_sql
from invenio.modules.upgrader.api import op


depends_on = [u'search_2014_07_21_facets_per_collection']


def info():
    """Info."""
    return "Add a autoincrement id in collection_field_fieldvalue table."""


def do_upgrade():
    """Implement your upgrades here."""
    # add column "id" in the table
    op.add_column('collection_field_fieldvalue',
                  db.Column('id', db.MediumInteger(9, unsigned=True),
                            nullable=False))

    # set all new ids
    records = run_sql("""SELECT id_collection, id_field, id_fieldvalue,
                      type, score, score_fieldvalue
                      FROM collection_field_fieldvalue AS cff
                      ORDER BY cff.id_collection, id_field, id_fieldvalue,
                      type, score, score_fieldvalue""")
    for index, rec in enumerate(records):
        sql = """UPDATE collection_field_fieldvalue
                 SET id = %%s
                 WHERE id_collection = %%s AND id_field = %%s
                 AND type = %%s AND score = %%s AND score_fieldvalue = %%s
                 AND id_fieldvalue %s
              """ % ('=%s' % (rec[2], ) if rec[2] is not None else 'is NULL', )
        run_sql(sql, (index + 1, rec[0], rec[1], rec[3], rec[4], rec[5]))

    # create new primary key with id
    op.create_primary_key('pk_collection_field_fieldvalue_id',
                          'collection_field_fieldvalue', ['id'])

    # set id as autoincrement
    op.alter_column('collection_field_fieldvalue', 'id',
                    existing_type=db.MediumInteger(9, unsigned=True),
                    existing_nullable=False, autoincrement=True)


def estimate():
    """Estimate running time of upgrade in seconds (optional)."""
    total = run_sql("SELECT count(*) FROM collection_field_fieldvalue")
    return int(float(total[0][0]) / 1000) + 1


def pre_upgrade():
    """Run pre-upgrade checks (optional)."""
    # Example of raising errors:
    # raise RuntimeError("Description of error 1", "Description of error 2")


def post_upgrade():
    """Run post-upgrade checks (optional)."""
    # Example of issuing warnings:
    # warnings.warn("A continuable error occurred")



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""Implement AST convertor to Elastic Search DSL."""

from invenio.base.globals import cfg
from invenio_query_parser.ast import (
    AndOp, KeywordOp, OrOp,
    NotOp, Keyword, Value,
    SingleQuotedValue,
    DoubleQuotedValue,
    RegexValue, RangeOp,
    ValueQuery, EmptyQuery,
    GreaterOp, GreaterEqualOp,
    LowerOp, LowerEqualOp
)

from invenio_query_parser.visitor import make_visitor


class ElasticSearchDSL(object):

    """Implement visitor to create Elastic Search DSL."""

    visitor = make_visitor()

    # pylint: disable=W0613,E0102

    def __init__(self):
        """Provide a dictinary mapping invenio keywords
        to elasticsearch fields as a list
        eg. {"author": ["author.last_name, author.first_name"]}
        """
        self.keyword_dict = cfg['SEARCH_ELASTIC_KEYWORD_MAPPING']

    def map_keyword_to_fields(self, keyword, mode='a'):
        """Convert keyword to keyword list for searches
           Map keyword to elasticsearch fields if needed
        """
        if self.keyword_dict:
            res = self.keyword_dict.get(keyword)
            if isinstance(res, dict):
                if mode in res:
                    res = res[mode]
                else:
                    raise RuntimeError(
                        'Not defined mapping for keyword "{keyword}" and '
                        'mode "{mode}"'.format(keyword=keyword, mode=mode)
                    )
            return res if res else [str(keyword)]
        return [str(keyword)]

    @visitor(AndOp)
    def visit(self, node, left, right):
        return {'bool': {'must': [left, right]}}

    @visitor(OrOp)
    def visit(self, node, left, right):
        return {'bool': {'should': [left, right]}}

    @visitor(NotOp)
    def visit(self, node, op):
        return {'bool': {'must_not': [op]}}

    @visitor(KeywordOp)
    def visit(self, node, left, right):
        if callable(right):
            keyword = self.map_keyword_to_fields(
                left, getattr(right, '__search_mode__', 'a')
            )
            return right(keyword)
        raise RuntimeError("Not supported second level operation.")

    @visitor(ValueQuery)
    def visit(self, node, op):
        return op(['global_fulltext'])

    @visitor(Keyword)
    def visit(self, node):
        return node.value

    @visitor(Value)
    def visit(self, node):
        def _f(keyword):
            return {
                'multi_match': {
                    'query': node.value,
                    'fields': keyword
                }
            }
        _f.__search_mode__ = 'a'
        return _f

    @visitor(SingleQuotedValue)
    def visit(self, node):
        def _f(keyword):
            return {
                'multi_match': {
                    'query': node.value,
                    'type': 'phrase',
                    'fields': keyword
                }
            }
        _f.__search_mode__ = 'p'
        return _f

    @visitor(DoubleQuotedValue)
    def visit(self, node):
        def _f(keyword):
            if (len(keyword) > 1):
                return {"bool":
                        {"should": [{"term": {k: str(node.value)}}
                                    for k in keyword]}}
            else:
                return {'term': {keyword[0]: node.value}}
        _f.__search_mode__ = 'e'
        return _f

    @visitor(RegexValue)
    def visit(self, node):
        def _f(keyword):
            if len(keyword) > 1:
                res = {"bool": {"should": []}}
                res["bool"]["should"] = [{'regexp': {k: node.value}}
                                         for k in keyword]
            elif keyword[0] != "_all":
                res = {'regexp': {keyword[0]: node.value}}
            else:
                raise RuntimeError("Not supported regex search for all fields")
            return res
        return _f

    @visitor(RangeOp)
    def visit(self, node, left, right):
        condition = {}
        if left:
            condition['gte'] = left(None)["multi_match"]["query"]
        if right:
            condition['lte'] = right(None)["multi_match"]["query"]

        def _f(keyword):
            if len(keyword) > 1:
                res = {"bool": {"should": []}}
                res["bool"]["should"] = [{'range': {k: condition}}
                                         for k in keyword]
            else:
                res = {'range': {keyword[0]: condition}}
            return res
        return _f

    @visitor(EmptyQuery)
    def visit(self, node):
        return {
            "match_all": {}
        }

    @staticmethod
    def _operators(node, condition):
        def _f(keyword):
            if len(keyword) > 1:
                res = {"bool": {"should": []}}
                res["bool"]["should"] = [{'range': {k: condition}}
                                         for k in keyword]
            else:
                res = {'range': {keyword[0]: condition}}
            return res
        return _f

    @visitor(GreaterOp)
    def visit(self, node, value_fn):
        condition = {"gt": value_fn(None)["multi_match"]["query"]}
        return self._operators(node, condition)

    @visitor(LowerOp)
    def visit(self, node, value_fn):
        condition = {"lt": value_fn(None)["multi_match"]["query"]}
        return self._operators(node, condition)

    @visitor(GreaterEqualOp)
    def visit(self, node, value_fn):
        condition = {"gte": value_fn(None)["multi_match"]["query"]}
        return self._operators(node, condition)

    @visitor(LowerEqualOp)
    def visit(self, node, value_fn):
        condition = {"lte": value_fn(None)["multi_match"]["query"]}
        return self._operators(node, condition)
    # pylint: enable=W0612,E0102



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2014, 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""Registries for search module."""

from flask_registry import RegistryError, ModuleAutoDiscoveryRegistry, \
    RegistryProxy
from werkzeug.utils import cached_property

from invenio.ext.registry import DictModuleAutoDiscoverySubRegistry, \
    ModuleAutoDiscoverySubRegistry
from invenio.utils.memoise import memoize

searchext = RegistryProxy('searchext', ModuleAutoDiscoveryRegistry,
                          'searchext')


class SearchServiceRegistry(ModuleAutoDiscoverySubRegistry):

    """Search Service Registry."""

    __required_plugin_API_version__ = "Search Service Plugin API 1.0"

    def register(self, item):
        """Check plugin version and instantiate search service plugin."""
        if item.__plugin_version__ != self.__required_plugin_API_version__:
            raise RegistryError(
                'Invalid plugin version {0} required {1}'.format(
                    item.__plugin_version__,
                    self.__required_plugin_API_version__
                ))
        service = getattr(item, item.__name__.split('.')[-1])
        return super(SearchServiceRegistry, self).register(service())

services = RegistryProxy('searchext.services', SearchServiceRegistry,
                         'services', registry_namespace=searchext)


class FacetsRegistry(DictModuleAutoDiscoverySubRegistry):

    """Registry for facets modules.

    Serves also modules sets and their configuration
    for specific collections.
    """

    def keygetter(self, key, original_value, new_value):
        """Compute the key for a value being registered.

        The key is the facet name stored in facet module.

        :param key: Key if provided by the user. Defaults to None.
        :param value: Value being registered. FacetBuilder object
        """
        return new_value.name

    def valuegetter(self, value):
        """Return FacetBuilder from inside the module.

        :param value: loaded python module with FacetBuilder instance
            stored in facet property
        """
        if self.facet_plugin_checker(value):
            return value.facet

    @classmethod
    def facet_plugin_checker(cls, plugin_code):
        """Handy function to check facet plugin.

        :param plugin_code: a module with facet definition - should have facet
            variable
        """
        from invenio_search.facet_builders import FacetBuilder
        if 'facet' in dir(plugin_code):
            candidate = getattr(plugin_code, 'facet')
            if isinstance(candidate, FacetBuilder):
                return candidate

    @memoize
    def get_facets_for_collection(self, collection_id):
        """Return facets set for a collection.

        :param collection_id: the collection id for requested facets set
        """
        from invenio_collections.models import FacetCollection
        facets_conf = FacetCollection.query\
            .filter(FacetCollection.id_collection == collection_id)\
            .order_by(FacetCollection.order)\
            .all()

        collection_facets = []
        for facet in facets_conf:
            if facet.facet_name not in self.keys():
                raise RegistryError(
                    'Facet %s is not available. Please check if the facet '
                    'is located in package specified in PACKAGES_EXCLUDE or '
                    'in PACKAGES_FACETS_EXCLUDE configuration.'
                    % facet.facet_name)
            collection_facets.append(self.get(facet.facet_name))

        return collection_facets

    @cached_property
    def default_facets(self):
        """Return default set of facets."""
        return self.get_facets_for_collection(1)

    def get_facets_config(self, collection, qid):
        """Return facet config for the collection.

        If no configuration found returns the default facets set.
        :param collection: Collection object facets matching which are returned
        :param qid: md5 hash of search parameters generated by
            get_search_query_id() from invenio_search.cache
        """
        if collection and self.get_facets_for_collection(collection.id):
            facets_set = self.get_facets_for_collection(collection.id)
        else:
            facets_set = self.default_facets

        return [facet.get_conf(collection=collection, qid=qid)
                for facet in facets_set]

facets = RegistryProxy('facets', FacetsRegistry, 'facets')

units = RegistryProxy(
    'searchext.units', DictModuleAutoDiscoverySubRegistry, 'units',
    keygetter=lambda key, value, new_value: value.__name__.split('.')[-1],
    valuegetter=lambda value: value.search_unit,
    registry_namespace=searchext,
)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""Unit tests for the elasticsearch AST walker"""

from invenio.testsuite import make_test_suite, run_test_suite, InvenioTestCase
from invenio_query_parser.ast import (
    AndOp, KeywordOp, OrOp, NotOp, Keyword, Value, SingleQuotedValue,
    DoubleQuotedValue, ValueQuery, RegexValue, RangeOp, EmptyQuery,
    GreaterOp, GreaterEqualOp, LowerOp, LowerEqualOp
)

from invenio_search.walkers.elasticsearch import ElasticSearchDSL


class TestElasticSearchWalker(InvenioTestCase):

    """Test transformations from the query-parser produced AST to the
       elasticsearch DSL query
    """

    def setUp(self):
        self.converter = ElasticSearchDSL()
        self.converter.keyword_dict = {"foo": ["test1", "test2"]}

    # Empty query
    def test_empty_query(self):
        self.assertEqual(EmptyQuery("").accept(self.converter), {
            "match_all": {
                }})

    # Value queries
    def test_value_query(self):
        self.assertEqual(ValueQuery(Value("bar")).accept(self.converter), {
            "multi_match": {
                "fields": ["global_fulltext"], "query": "bar"}
            })

    def test_single_quoted_value(self):
        tree = ValueQuery(SingleQuotedValue("bar"))
        self.assertEqual(tree.accept(self.converter), {
            "multi_match": {
                "fields": ["global_fulltext"],
                "query": "bar",
                "type": "phrase"
            }
        })

    def test_double_quoted_value(self):
        tree = ValueQuery(DoubleQuotedValue("bar"))
        self.assertEqual(tree.accept(self.converter), {
            "term": {"global_fulltext": "bar"}
        })

    def test_regex_value(self):
        tree = ValueQuery(RegexValue('^E.*s$'))
        self.assertEqual(tree.accept(self.converter), {
            "regexp": {"global_fulltext": '^E.*s$'}
        })

    # Key-value queries
    def test_key_val_query(self):
        tree = KeywordOp(Keyword('foo'), Value('bar'))
        self.assertEqual(tree.accept(self.converter), {
            "multi_match": {
                "fields": ["test1", "test2"],
                "query": "bar"
            }
        })

    def test_key_val_single_quote(self):
        tree = KeywordOp(Keyword('foo'), SingleQuotedValue('bar'))
        self.assertEqual(tree.accept(self.converter), {
            "multi_match": {
                "fields": ["test1", "test2"],
                "query": "bar",
                "type": "phrase"
            }
        })

    def test_key_val_double_quote(self):
        tree = KeywordOp(Keyword('foo'), DoubleQuotedValue('bar'))
        self.assertEqual(tree.accept(self.converter), {
            "bool": {
                "should": [
                    {"term": {"test1": "bar"}},
                    {"term": {"test2": "bar"}}]
            }
        })

    def test_range_op(self):
        tree = KeywordOp(Keyword('year'),
                         RangeOp(Value('2000'), Value('2012')))
        self.assertEqual(tree.accept(self.converter), {
            "range": {
                "year": {
                    "gte": "2000",
                    "lte": "2012",
                }
            }
        })

    def test_key_regex(self):
        tree = KeywordOp(Keyword('boo'), RegexValue('bar'))
        print tree.accept(self.converter)
        self.assertEqual(tree.accept(self.converter), {
            "regexp": {"boo": "bar"}
        })

    # Combined queries - boolean opeations
    def test_and_query(self):
        tree = AndOp(KeywordOp(Keyword('boo'), Value('bar')),
                     ValueQuery(Value('baz')))
        self.assertEqual(tree.accept(self.converter), {
            "bool": {
                "must": [
                    {
                        "multi_match": {
                            "fields": ["boo"],
                            "query": "bar",
                        }
                    },
                    {
                        "multi_match": {
                            "fields": ["global_fulltext"],
                            "query": "baz",
                        }
                    }
                ]
            }
        })

    def test_or_query(self):
        tree = OrOp(KeywordOp(Keyword('boo'), Value('bar')),
                    ValueQuery(Value('baz')))
        self.assertEqual(tree.accept(self.converter), {
            "bool": {
                "should": [
                    {
                        "multi_match": {
                            "fields": ["boo"],
                            "query": "bar",
                        }
                    },
                    {
                        "multi_match": {
                            "fields": ["global_fulltext"],
                            "query": "baz",
                        }
                    }
                ]
            }
        })

    def test_not_query(self):
        tree = AndOp(KeywordOp(Keyword('boo'), Value('bar')),
                     NotOp(KeywordOp(Keyword('boo'), Value('bar'))))
        self.assertEqual(tree.accept(self.converter), {
            "bool": {
                "must": [
                    {
                        "multi_match": {
                            "fields": ["boo"],
                            "query": "bar",
                        }
                    },
                    {
                        "bool": {
                            "must_not": [
                                {
                                    "multi_match": {
                                        "fields": ["boo"],
                                        "query": "bar",
                                    }
                                }
                            ]
                        }
                    }
                ]
            }
        })

    def test_combined_bool(self):
        tree = OrOp(AndOp(AndOp(ValueQuery(Value('aaa')),
                                ValueQuery(Value('bbb'))),
                          NotOp(ValueQuery(Value('ccc')))),
                    ValueQuery(Value('ddd')))
        ll = {
                "multi_match": {
                    "fields": ["global_fulltext"],
                    "query": "ddd"
                }
             }
        rr = {
                "bool": {
                    "must": [
                        {
                            "multi_match": {
                                "fields": ["global_fulltext"],
                                "query": "aaa"
                            }
                        },
                        {
                            "multi_match": {
                                "fields": ["global_fulltext"],
                                "query": "bbb"
                            }
                        }
                    ]
                }
             }
        rl = {
                "bool": {
                    "must_not": [
                        {
                            "multi_match": {
                                "fields": ["global_fulltext"],
                                "query": "ccc"
                            }
                        }
                    ]
                }
             }
        r = {
                "bool": {
                    "must": [rr, rl]
                }
            }
        d = {
                "bool": {
                    "should": [r, ll]
                }
            }
        self.assertEqual(tree.accept(self.converter), d)

    # Operators
    def test_greater_op(self):
        tree = KeywordOp(Keyword('date'), GreaterOp(Value('1984')))
        self.assertEqual(tree.accept(self.converter), {
            "range": {
                "date": {"gt": "1984"}
            }
        })

    def test_lower_op(self):
        tree = KeywordOp(Keyword('date'), LowerOp(Value('1984')))
        self.assertEqual(tree.accept(self.converter), {
            "range": {
                "date": {"lt": "1984"}
            }
        })

    def test_gte_op(self):
        tree = KeywordOp(Keyword('date'), GreaterEqualOp(Value('1984')))
        self.assertEqual(tree.accept(self.converter), {
            "range": {
                "date": {"gte": "1984"}
            }
        })

    def test_lte_op(self):
        tree = KeywordOp(Keyword('date'), LowerEqualOp(Value('1984')))
        self.assertEqual(tree.accept(self.converter), {
            "range": {
                "date": {"lte": "1984"}
            }
        })

TEST_SUITE = make_test_suite(TestElasticSearchWalker)

if __name__ == "__main__":
    run_test_suite(TEST_SUITE)




# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2012, 2013, 2014, 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""WebSearch User Settings."""

from flask import url_for
from flask_login import current_user
from invenio.base.i18n import _
from invenio.ext.sqlalchemy import db
from invenio.ext.template import render_template_to_string
from .models import UserQuery
from .forms import WebSearchUserSettingsForm
from invenio.modules.dashboard.settings import Settings, UserSettingsStorage


class WebSearchSettings(Settings):

    """WebSearch User Settings."""

    keys = ['rg', 'websearch_hotkeys', 'c', 'of']
    form_builder = WebSearchUserSettingsForm
    storage_builder = UserSettingsStorage

    def __init__(self):
        super(WebSearchSettings, self).__init__()
        self.icon = 'search'
        self.title = _('Searches')
        self.view = '/youralerts/display'  # FIXME url_for('youralerts.index')
        self.edit = url_for('webaccount.edit', name=self.name)

    def widget(self):
        """Display search settings widget."""
        uid = current_user.get_id()
        queries = db.session.query(db.func.count(UserQuery.id_query)).filter(
            UserQuery.id_user == uid
            ).scalar()

        template = """
{{ _('You have made %(x_num_queries)d queries. A detailed list is available with a possibility to
(a) view search results and (b) subscribe to an automatic email alerting service
for these queries.', x_num_queries=queries) }}
"""

        return render_template_to_string(template, _from_string=True,
                                         queries=queries)

    widget.size = 4

    @property
    def is_authorized(self):
        return not current_user.is_guest

# Compulsory plugin interface
settings = WebSearchSettings
#__all__ = ['WebSearchSettings']



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""Subject search unit."""

from invenio_search.searchext.engines.native import default_search_unit


def search_unit(p, f, m, wl=None):
    """Search subject always in phrase index."""
    return default_search_unit(p, f, 'a', wl)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2010, 2011, 2012, 2013,
#               2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""Invenio Search Engine config parameters."""

from __future__ import unicode_literals

SEARCH_QUERY_PARSER = 'invenio_query_parser.parser:Main'

SEARCH_QUERY_WALKERS = [
    'invenio_query_parser.walkers.pypeg_to_ast:PypegConverter',
]

# SEARCH_QUERY_ENHANCERS -- a comma separated list of strings. Each string is a
# function that is applied to the AST generated by the parser and enhances the
# query tree
SEARCH_QUERY_ENHANCERS = [
    # 'invenio_search.enhancers.cache_results.apply',
    'invenio_search.enhancers.collection_filter.apply',
    # 'invenio_search.enhancers.facet_filter.apply',
]

# SEARCH_WALKERS -- a comma separated list of strings. Each string is
# a AST visitor class.
SEARCH_WALKERS = [
    # 'invenio_search.walkers.search_unit:SearchUnit',
    'invenio_search.walkers.elasticsearch:ElasticSearchDSL',
]

# do we want experimental features? (0=no, 1=yes)
CFG_EXPERIMENTAL_FEATURES = 0

# CFG_WEBSEARCH_IDXPAIRS_FIELDS -- a comma separated list of index
# fields. This list contains all the index fields on which exact
# phrase search should use idxPairs tables.
CFG_WEBSEARCH_IDXPAIRS_FIELDS = ['global', 'abstract', 'title', 'caption']

# CFG_WEBSEARCH_IDXPAIRS_EXACT_SEARCH -- if true, it will eliminate
# all the false positives when using the word pairs for search.
# (Example: `foo bar baz' being search as `foo bar' and `bar baz' may
# lead to false positives if there is no second-pass.)  FIXME: we
# need this to be defined per index if we want to eliminate
# single-quoted vs double-quoted search difference, e.g. False for
# title search, but True for report number search.
CFG_WEBSEARCH_IDXPAIRS_EXACT_SEARCH = False

# Maximum number of collections to be displayed on the search results
# page. All the rest of the collections will be hidden by a
# "See more collections" link.
CFG_WEBSEARCH_RESULTS_OVERVIEW_MAX_COLLS_TO_PRINT = 10

# Prefix used for search results cache.
CFG_SEARCH_RESULTS_CACHE_PREFIX = "search_results::"

# CERN Site hack
# CFG_WEBSEARCH_SEARCH_WITHIN = ['title',
#                                'author',
#                                'abstract',
#                                'report number,
#                                'year']

CFG_WEBSEARCH_SEARCH_WITHIN = ['title',
                               'author',
                               'abstract',
                               'keyword',
                               'report number',
                               'journal',
                               'year',
                               'fulltext',
                               'reference']


CFG_WEBSEACH_MATCHING_TYPES = [
    {
        'code': 'a',
        'title': "all of the words",
        'order': 1,
        'tokenize': """
            var vals = val.split(' '),
                result = $.map(vals, function(e) {
              return f+e;
            }).join(' AND ');
            if (vals.length > 1) {
                result = '(' + result +')';
            }
            return result;
        """
    },
    {
        'code': 'o',
        'title': "any of the words",
        'order': 2,
        'tokenize': """
            var vals = val.split(' '),
                result = $.map(vals, function(e) {
              return f+e;
            }).join(' OR ');
            if (vals.length > 1) {
                result = '(' + result +')';
            }
            return result;
        """
    },
    {
        'code': 'e',
        'title': "exact phrase",
        'order': 3,
        'tokenize': """
            return f+'"'+val+'"';
        """
    },
    {
        'code': 'p',
        'title': "partial phrase",
        'order': 4,
        'tokenize': """
            return f+"'"+val+"'";
        """
    },
    {
        'code': 'r',
        'title': "regular expression",
        'order': 5,
        'tokenize': """
            return f+'/'+val+'/';
        """
    }
]

# CFG_WEBSEARCH_COLLECTION_NAMES_SEARCH -- decides whether search for
# collection name is enabled (1), disabled (-1) or enabled only for
# the home collection (0), enabled for all collections including
# those not attached to the collection tree (2). This requires the
# CollectionNameSearchService search services to be enabled.)
CFG_WEBSEARCH_COLLECTION_NAMES_SEARCH = 0

# CFG_WEBSEARCH_MAX_RECORDS_REFERSTO -- in order to limit denial of service
# attacks the total number of records for which we look for incoming citations
# (all the records that have a reference/citation to the specified records)
# will be limited to this number. This does not limit the number of records in
# the result.
CFG_WEBSEARCH_MAX_RECORDS_REFERSTO = 50000

# CFG_WEBSEARCH_MAX_RECORDS_CITEDBY -- in order to limit denial of service
# attacks the total number of records for which we look for outgoing citations
# (all the records referenced/cited by the specified records) will be limited
# to this number. This does not limit the number of records in the result.
CFG_WEBSEARCH_MAX_RECORDS_CITEDBY = 50000

# SEARCH_ELASTIC_KEYWORD_MAPPING -- this variable holds a dictionary to map
# invenio keywords to elasticsearch fields
SEARCH_ELASTIC_KEYWORD_MAPPING = {
    "author": {
        'a': ["main_entry_personal_name.personal_name",
              "added_entry_personal_name.personal_name"],
        'p': ["main_entry_personal_name.personal_name",
              "added_entry_personal_name.personal_name"],
        'e': ['authors.raw'],
    },
    "collection": ["_collections"],
    "title": ["title_statement.title"],
    "980": [
        "collections.primary",
        "collections.secondary",
        "collections.deleted",
    ],
    "980__a": ["collections.primary"],
    "980__b": ["collections.secondary"],
    "542__l": ["information_relating_to_copyright_status.copyright_status"],
}



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2012, 2013, 2014 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

from invenio.config import CFG_SITE_NAME
from fixture import DataSet


class CollectionData(DataSet):
    class siteCollection:
        id = 1
        name = CFG_SITE_NAME
        dbquery = None


class FieldData(DataSet):

    class Field_1:
        code = u'anyfield'
        id = 1
        name = u'any field'

    class Field_2:
        code = u'title'
        id = 2
        name = u'title'

    class Field_3:
        code = u'author'
        id = 3
        name = u'author'

    class Field_4:
        code = u'abstract'
        id = 4
        name = u'abstract'

    class Field_5:
        code = u'keyword'
        id = 5
        name = u'keyword'

    class Field_6:
        code = u'reportnumber'
        id = 6
        name = u'report number'

    class Field_7:
        code = u'subject'
        id = 7
        name = u'subject'

    class Field_8:
        code = u'reference'
        id = 8
        name = u'reference'

    class Field_9:
        code = u'fulltext'
        id = 9
        name = u'fulltext'

    class Field_10:
        code = u'collection'
        id = 10
        name = u'collection'

    class Field_11:
        code = u'division'
        id = 11
        name = u'division'

    class Field_12:
        code = u'year'
        id = 12
        name = u'year'

    class Field_13:
        code = u'experiment'
        id = 13
        name = u'experiment'

    class Field_14:
        code = u'recid'
        id = 14
        name = u'record ID'

    class Field_15:
        code = u'isbn'
        id = 15
        name = u'isbn'

    class Field_16:
        code = u'issn'
        id = 16
        name = u'issn'

    class Field_17:
        code = u'coden'
        id = 17
        name = u'coden'

    #class Field_18:
    #    code = u'doi'
    #    id = 18
    #    name = u'doi'

    class Field_19:
        code = u'journal'
        id = 19
        name = u'journal'

    class Field_20:
        code = u'collaboration'
        id = 20
        name = u'collaboration'

    class Field_21:
        code = u'affiliation'
        id = 21
        name = u'affiliation'

    class Field_22:
        code = u'exactauthor'
        id = 22
        name = u'exact author'

    class Field_23:
        code = u'datecreated'
        id = 23
        name = u'date created'

    class Field_24:
        code = u'datemodified'
        id = 24
        name = u'date modified'

    class Field_25:
        code = u'refersto'
        id = 25
        name = u'refers to'

    class Field_26:
        code = u'citedby'
        id = 26
        name = u'cited by'

    class Field_27:
        code = u'caption'
        id = 27
        name = u'caption'

    class Field_28:
        code = u'firstauthor'
        id = 28
        name = u'first author'

    class Field_29:
        code = u'exactfirstauthor'
        id = 29
        name = u'exact first author'

    class Field_30:
        code = u'authorcount'
        id = 30
        name = u'author count'

    class Field_31:
        code = u'rawref'
        id = 31
        name = u'reference to'

    class Field_32:
        code = u'exacttitle'
        id = 32
        name = u'exact title'

    class Field_33:
        code = u'authorityauthor'
        id = 33
        name = u'authority author'

    class Field_34:
        code = u'authorityinstitute'
        id = 34
        name = u'authority institution'

    class Field_35:
        code = u'authorityjournal'
        id = 35
        name = u'authority journal'

    class Field_36:
        code = u'authoritysubject'
        id = 36
        name = u'authority subject'

    class Field_37:
        code = u'itemcount'
        id = 37
        name = u'item count'

    class Field_38:
        code = u'filetype'
        id = 38
        name = u'file type'

    class Field_39:
        code = u'miscellaneous'
        id = 39
        name = u'miscellaneous'

    class Field_40:
        code = u'referstoexcludingselfcites'
        id = 40
        name = u'refers to excluding self cites'

    class Field_41:
        code = u'citedbyexcludingselfcites'
        id = 41
        name = u'cited by excluding self cites'

    class Field_42:
        code = u'cataloguer'
        id = 42
        name = u'cataloguer nickname'

    class Field_43:
        code = u'filename'
        id = 43
        name = u'file name'

    class Field_44:
        code = u'tag'
        id = 44
        name = u'tag'


class TagData(DataSet):

    class Tag_1:
        id = 1
        value = u'100__a'
        recjson_value = u'main_entry_personal_name.personal_name'
        name = u'first author name'

    class Tag_2:
        id = 2
        value = u'700__a'
        recjson_value = u'added_entry_personal_name.personal_name'
        name = u'additional author name'

    class Tag_3:
        id = 3
        value = u'245__%'
        recjson_value = u'title_statement'
        name = u'main title'

    class Tag_4:
        id = 4
        value = u'246__%'
        recjson_value = u'varying_form_of_title'
        name = u'additional title'

    class Tag_5:
        id = 5
        value = u'520__%'
        recjson_value = u'abstract'
        name = u'abstract'

    class Tag_6:
        id = 6
        value = u'6531_a'
        recjson_value = u'index_term_uncontrolled.uncontrolled_term'
        name = u'keyword'

    class Tag_7:
        id = 7
        value = u'037__a'
        recjson_value = u'source_of_acquisition.stock_number'
        name = u'primary report number'

    class Tag_8:
        id = 8
        value = u'088__a'
        recjson_value = u'report_number.report_number'
        name = u'additional report number'

    class Tag_9:
        id = 9
        value = u'909C0r'
        recjson_value = u'added_report_number'
        name = u'added report number'

    class Tag_10:
        id = 10
        value = u'999C5%'
        recjson_value = u'reference'
        name = u'reference'

    class Tag_11:
        id = 11
        value = u''
        recjson_value = u'_collections'
        name = u'collection identifier'

    class Tag_12:
        id = 12
        value = u'65017a'
        recjson_value = u'subject_added_entry_topical_term.topical_term_or_geographic_name_entry_element'
        name = u'main subject'

    class Tag_13:
        id = 13
        value = u'65027a'
        recjson_value = u'subject_additional.term'
        name = u'additional subject'

    class Tag_14:
        id = 14
        value = u'909C0p'
        recjson_value = u'division'
        name = u'division'

    class Tag_15:
        id = 15
        value = u'909C0y'
        recjson_value = u'year'
        name = u'year'

    class Tag_16:
        id = 16
        value = u'00%'
        recjson_value = u'agency_code,recid,version_id'
        name = u'00x'

    class Tag_17:
        id = 17
        value = u'01%'
        recjson_value = u''
        name = u'01x'

    class Tag_18:
        id = 18
        value = u'02%'
        recjson_value = u'oai,doi,isbn,issn,isn'
        name = u'02x'

    class Tag_19:
        id = 19
        value = u'03%'
        recjson_value = u'code_designation,system_control_number'
        name = u'03x'

    class Tag_20:
        id = 20
        value = u'04%'
        recjson_value = u'language,publishing_country'
        name = u'lang'

    class Tag_21:
        id = 21
        value = u'05%'
        recjson_value = u'library_of_congress_call_number'
        name = u'05x'

    class Tag_22:
        id = 22
        value = u'06%'
        recjson_value = u''
        name = u'06x'

    class Tag_23:
        id = 23
        value = u'07%'
        recjson_value = u''
        name = u'07x'

    class Tag_24:
        id = 24
        value = u'08%'
        recjson_value = u'dewey_decimal_classification_number,other_report_number,report_number,udc'
        name = u'08x'

    class Tag_25:
        id = 25
        value = u'09%'
        recjson_value = u''
        name = u'09x'

    class Tag_26:
        id = 26
        value = u'10%'
        recjson_value = u'authors'
        name = u'10x'

    class Tag_27:
        id = 27
        value = u'11%'
        recjson_value = u'corporate_name[0],meeting_name[0]'
        name = u'11x'

    class Tag_28:
        id = 28
        value = u'12%'
        recjson_value = u''
        name = u'12x'

    class Tag_29:
        id = 29
        value = u'13%'
        recjson_value = u''
        name = u'13x'

    class Tag_30:
        id = 30
        value = u'14%'
        recjson_value = u'main_title_statement,chronological_term'
        name = u'14x'

    class Tag_31:
        id = 31
        value = u'15%'
        recjson_value = u''
        name = u'15x'

    class Tag_32:
        id = 32
        value = u'16%'
        recjson_value = u''
        name = u'16x'

    class Tag_33:
        id = 33
        value = u'17%'
        recjson_value = u''
        name = u'17x'

    class Tag_34:
        id = 34
        value = u'18%'
        recjson_value = u'medium'
        name = u'18x'

    class Tag_35:
        id = 35
        value = u'19%'
        recjson_value = u''
        name = u'19x'

    class Tag_36:
        id = 36
        value = u'20%'
        recjson_value = u''
        name = u'20x'

    class Tag_37:
        id = 37
        value = u'21%'
        recjson_value = u'abbreviated_title'
        name = u'21x'

    class Tag_38:
        id = 38
        value = u'22%'
        recjson_value = u'title_key'
        name = u'22x'

    class Tag_39:
        id = 39
        value = u'23%'
        recjson_value = u''
        name = u'23x'

    class Tag_40:
        id = 40
        value = u'24%'
        recjson_value = u'title_additional,title_other,title,title_parallel,title_translation'
        name = u'24x'

    class Tag_41:
        id = 41
        value = u'25%'
        recjson_value = u'edition_statement'
        name = u'25x'

    class Tag_42:
        id = 42
        value = u'26%'
        recjson_value = u'imprint,prepublication'
        name = u'internal'

    class Tag_43:
        id = 43
        value = u'27%'
        recjson_value = u'address'
        name = u'27x'

    class Tag_44:
        id = 44
        value = u'28%'
        recjson_value = u''
        name = u'28x'

    class Tag_45:
        id = 45
        value = u'29%'
        recjson_value = u''
        name = u'29x'

    class Tag_46:
        id = 46
        value = u'30%'
        recjson_value = u'physical_description'
        name = u'pages'

    class Tag_47:
        id = 47
        value = u'31%'
        recjson_value = u'current_publication_frequency'
        name = u'31x'

    class Tag_48:
        id = 48
        value = u'32%'
        recjson_value = u''
        name = u'32x'

    class Tag_49:
        id = 49
        value = u'33%'
        recjson_value = u'content_type'
        name = u'33x'

    class Tag_50:
        id = 50
        value = u'34%'
        recjson_value = u'medium'
        name = u'34x'

    class Tag_51:
        id = 51
        value = u'35%'
        recjson_value = u''
        name = u'35x'

    class Tag_52:
        id = 52
        value = u'36%'
        recjson_value = u''
        name = u'36x'

    class Tag_53:
        id = 53
        value = u'37%'
        recjson_value = u''
        name = u'37x'

    class Tag_54:
        id = 54
        value = u'38%'
        recjson_value = u''
        name = u'38x'

    class Tag_55:
        id = 55
        value = u'39%'
        recjson_value = u''
        name = u'39x'

    class Tag_56:
        id = 56
        value = u'40%'
        recjson_value = u''
        name = u'40x'

    class Tag_57:
        id = 57
        value = u'41%'
        recjson_value = u''
        name = u'41x'

    class Tag_58:
        id = 58
        value = u'42%'
        recjson_value = u''
        name = u'42x'

    class Tag_59:
        id = 59
        value = u'43%'
        recjson_value = u''
        name = u'43x'

    class Tag_60:
        id = 60
        value = u'44%'
        recjson_value = u''
        name = u'44x'

    class Tag_61:
        id = 61
        value = u'45%'
        recjson_value = u''
        name = u'45x'

    class Tag_62:
        id = 62
        value = u'46%'
        recjson_value = u''
        name = u'46x'

    class Tag_63:
        id = 63
        value = u'47%'
        recjson_value = u''
        name = u'47x'

    class Tag_64:
        id = 64
        value = u'48%'
        recjson_value = u''
        name = u'48x'

    class Tag_65:
        id = 65
        value = u'49%'
        recjson_value = u'series'
        name = u'series'

    class Tag_66:
        id = 66
        value = u'50%'
        recjson_value = u'comment,dissertation_note,restriction_access,other_restriction_access'
        name = u'50x'

    class Tag_67:
        id = 67
        value = u'51%'
        recjson_value = u'time_and_place_of_event_note'
        name = u'51x'

    class Tag_68:
        id = 68
        value = u'52%'
        recjson_value = u'abstract'
        name = u'52x'

    class Tag_69:
        id = 69
        value = u'53%'
        recjson_value = u'funding_info'
        name = u'53x'

    class Tag_70:
        id = 70
        value = u'54%'
        recjson_value = u'copyright_information,language_note,license,source_of_acquisition'
        name = u'54x'

    class Tag_71:
        id = 71
        value = u'55%'
        recjson_value = u'cumulative_index'
        name = u'55x'

    class Tag_72:
        id = 72
        value = u'56%'
        recjson_value = u''
        name = u'56x'

    class Tag_73:
        id = 73
        value = u'57%'
        recjson_value = u''
        name = u'57x'

    class Tag_74:
        id = 74
        value = u'58%'
        recjson_value = u'action_note,source_of_description'
        name = u'58x'

    class Tag_75:
        id = 75
        value = u'59%'
        recjson_value = u'abstract_french,cern_bookshop_statistics,copyright,internal_notes,observation_french,slac_note,type'
        name = u'summary'

    class Tag_76:
        id = 76
        value = u'60%'
        recjson_value = u''
        name = u'60x'

    class Tag_77:
        id = 77
        value = u'61%'
        recjson_value = u''
        name = u'61x'

    class Tag_78:
        id = 78
        value = u'62%'
        recjson_value = u''
        name = u'62x'

    class Tag_79:
        id = 79
        value = u'63%'
        recjson_value = u''
        name = u'63x'

    class Tag_80:
        id = 80
        value = u'64%'
        recjson_value = u'publisher'
        name = u'64x'

    class Tag_81:
        id = 81
        value = u'65%'
        recjson_value = u'keywords'
        name = u'65x'

    class Tag_82:
        id = 82
        value = u'66%'
        recjson_value = u''
        name = u'66x'

    class Tag_83:
        id = 83
        value = u'67%'
        recjson_value = u'administrative_history,source_data_found'
        name = u'67x'

    class Tag_84:
        id = 84
        value = u'68%'
        recjson_value = u'public_general_note'
        name = u'68x'

    class Tag_85:
        id = 85
        value = u'69%'
        recjson_value = u'accelerator_experiment,cataloguer_info,classification_terms,observation,subject_indicator,thesaurus_terms,lexi_keyword'
        name = u'subject'

    class Tag_86:
        id = 86
        value = u'70%'
        recjson_value = u'contributor'
        name = u'70x'

    class Tag_87:
        id = 87
        value = u'71%'
        recjson_value = u'corporate_name[n],meeting_name[n]'
        name = u'71x'

    class Tag_88:
        id = 88
        value = u'72%'
        recjson_value = u'author_archive'
        name = u'author-ad'

    class Tag_89:
        id = 89
        value = u'73%'
        recjson_value = u''
        name = u'73x'

    class Tag_90:
        id = 90
        value = u'74%'
        recjson_value = u''
        name = u'74x'

    class Tag_91:
        id = 91
        value = u'75%'
        recjson_value = u''
        name = u'75x'

    class Tag_92:
        id = 92
        value = u'76%'
        recjson_value = u''
        name = u'76x'

    class Tag_93:
        id = 93
        value = u'77%'
        recjson_value = u'publication_info'
        name = u'77x'

    class Tag_94:
        id = 94
        value = u'78%'
        recjson_value = u''
        name = u'78x'

    class Tag_95:
        id = 95
        value = u'79%'
        recjson_value = u''
        name = u'79x'

    class Tag_96:
        id = 96
        value = u'80%'
        recjson_value = u''
        name = u'80x'

    class Tag_97:
        id = 97
        value = u'81%'
        recjson_value = u''
        name = u'81x'

    class Tag_98:
        id = 98
        value = u'82%'
        recjson_value = u''
        name = u'82x'

    class Tag_99:
        id = 99
        value = u'83%'
        recjson_value = u''
        name = u'83x'

    class Tag_100:
        id = 100
        value = u'84%'
        recjson_value = u''
        name = u'84x'

    class Tag_101:
        id = 101
        value = u'85%'
        recjson_value = u'location,email,email_message,electronic_location'
        name = u'electr'

    class Tag_102:
        id = 102
        value = u'86%'
        recjson_value = u''
        name = u'86x'

    class Tag_103:
        id = 103
        value = u'87%'
        recjson_value = u''
        name = u'87x'

    class Tag_104:
        id = 104
        value = u'88%'
        recjson_value = u''
        name = u'88x'

    class Tag_105:
        id = 105
        value = u'89%'
        recjson_value = u''
        name = u'89x'

    class Tag_106:
        id = 106
        value = u'90%'
        recjson_value = u'journal_info'
        name = u'publication'

    class Tag_107:
        id = 107
        value = u'91%'
        recjson_value = u'status_week,citation,universal_decimal_classification'
        name = u'pub-conf-cit'

    class Tag_108:
        id = 108
        value = u'92%'
        recjson_value = u'other_institution'
        name = u'92x'

    class Tag_109:
        id = 109
        value = u'93%'
        recjson_value = u''
        name = u'93x'

    class Tag_110:
        id = 110
        value = u'94%'
        recjson_value = u''
        name = u'94x'

    class Tag_111:
        id = 111
        value = u'95%'
        recjson_value = u''
        name = u'95x'

    class Tag_112:
        id = 112
        value = u'96%'
        recjson_value = u'aleph_linking_page,base,cataloguer_info,item,owner'
        name = u'catinfo'

    class Tag_113:
        id = 113
        value = u'97%'
        recjson_value = u'system_number'
        name = u'97x'

    class Tag_114:
        id = 114
        value = u'98%'
        recjson_value = u''
        name = u'98x'

    class Tag_115:
        id = 115
        value = u'8564_u'
        recjson_value = u'url.url'
        name = u'url'

    class Tag_116:
        id = 116
        value = u'909C0e'
        recjson_value = u'accelerator_experiment.experiment'
        name = u'experiment'

    class Tag_117:
        id = 117
        value = u'001'
        recjson_value = u'recid'
        name = u'record ID'

    class Tag_118:
        id = 118
        value = u'020__a'
        recjson_value = u'isbn'
        name = u'isbn'

    class Tag_119:
        id = 119
        value = u'022__a'
        recjson_value = u'issn'
        name = u'issn'

    class Tag_120:
        id = 120
        value = u'030__a'
        recjson_value = u''
        name = u'coden'

    class Tag_121:
        id = 121
        value = u'909C4a'
        recjson_value = u'journal_info.doi'
        name = u'doi'

    class Tag_122:
        id = 122
        value = u'850%'
        recjson_value = u''
        name = u'850x'

    class Tag_123:
        id = 123
        value = u'851%'
        recjson_value = u''
        name = u'851x'

    class Tag_124:
        id = 124
        value = u'852%'
        recjson_value = u'location'
        name = u'852x'

    class Tag_125:
        id = 125
        value = u'853%'
        recjson_value = u''
        name = u'853x'

    class Tag_126:
        id = 126
        value = u'854%'
        recjson_value = u''
        name = u'854x'

    class Tag_127:
        id = 127
        value = u'855%'
        recjson_value = u''
        name = u'855x'

    class Tag_128:
        id = 128
        value = u'857%'
        recjson_value = u''
        name = u'857x'

    class Tag_129:
        id = 129
        value = u'858%'
        recjson_value = u''
        name = u'858x'

    class Tag_130:
        id = 130
        value = u'859%'
        recjson_value = u'email_message'
        name = u'859x'

    class Tag_131:
        id = 131
        value = u'909C4%'
        recjson_value = u'journal_info'
        name = u'journal'

    class Tag_132:
        id = 132
        value = u'710__g'
        recjson_value = u'corporate_name[n].collaboration'
        name = u'collaboration'

    class Tag_133:
        id = 133
        value = u'100__u'
        recjson_value = u'authors[0].affiliation'
        name = u'first author affiliation'

    class Tag_134:
        id = 134
        value = u'700__u'
        recjson_value = u'contributor.affiliation'
        name = u'additional author affiliation'

    class Tag_135:
        id = 135
        value = u'8564_y'
        recjson_value = u'url.description'
        name = u'caption'

    class Tag_136:
        id = 136
        value = u'909C4c'
        recjson_value = u'journal_info.pagination'
        name = u'journal page'

    class Tag_137:
        id = 137
        value = u'909C4p'
        recjson_value = u'journal_info.title'
        name = u'journal title'

    class Tag_138:
        id = 138
        value = u'909C4v'
        recjson_value = u'journal_info.volume'
        name = u'journal volume'

    class Tag_139:
        id = 139
        value = u'909C4y'
        recjson_value = u'journal_info.year'
        name = u'journal year'

    class Tag_140:
        id = 140
        value = u'500__a'
        recjson_value = u'commnt'
        name = u'comment'

    class Tag_141:
        id = 141
        value = u'245__a'
        recjson_value = u'title.title'
        name = u'title'

    class Tag_142:
        id = 142
        value = u'245__a'
        recjson_value = u''
        name = u'main abstract'

    class Tag_143:
        id = 143
        value = u'595__a'
        recjson_value = u'internal_notes.internal_note'
        name = u'internal notes'

    class Tag_144:
        id = 144
        value = u'787%'
        recjson_value = u''
        name = u'other relationship entry'

    class Tag_146:
        id = 146
        value = u'400__a'
        recjson_value = u''
        name = u'authority: alternative personal name'

    class Tag_148:
        id = 148
        value = u'110__a'
        recjson_value = u'corporate_name[0].name'
        name = u'authority: organization main name'

    class Tag_149:
        id = 149
        value = u'410__a'
        recjson_value = u''
        name = u'organization alternative name'

    class Tag_150:
        id = 150
        value = u'510__a'
        recjson_value = u''
        name = u'organization main from other record'

    class Tag_151:
        id = 151
        value = u'130__a'
        recjson_value = u''
        name = u'authority: uniform title'

    class Tag_152:
        id = 152
        value = u'430__a'
        recjson_value = u''
        name = u'authority: uniform title alternatives'

    class Tag_153:
        id = 153
        value = u'530__a'
        recjson_value = u''
        name = u'authority: uniform title from other record'

    class Tag_154:
        id = 154
        value = u'150__a'
        recjson_value = u''
        name = u'authority: subject from other record'

    class Tag_155:
        id = 155
        value = u'450__a'
        recjson_value = u''
        name = u'authority: subject alternative name'

    class Tag_156:
        id = 156
        value = u'450__a'
        recjson_value = u''
        name = u'authority: subject main name'

    class Tag_157:
        id = 157
        value = u'031%'
        recjson_value = u''
        name = u'031x'

    class Tag_158:
        id = 158
        value = u'032%'
        recjson_value = u''
        name = u'032x'

    class Tag_159:
        id = 159
        value = u'033%'
        recjson_value = u'code_designation'
        name = u'033x'

    class Tag_160:
        id = 160
        value = u'034%'
        recjson_value = u''
        name = u'034x'

    class Tag_161:
        id = 161
        value = u'035%'
        recjson_value = u'system_control_number'
        name = u'035x'

    class Tag_162:
        id = 162
        value = u'036%'
        recjson_value = u''
        name = u'036x'

    class Tag_163:
        id = 163
        value = u'037%'
        recjson_value = u'primary_report_number'
        name = u'037x'

    class Tag_164:
        id = 164
        value = u'038%'
        recjson_value = u''
        name = u'038x'

    class Tag_165:
        id = 165
        value = u'080%'
        recjson_value = u'udc'
        name = u'080x'

    class Tag_166:
        id = 166
        value = u'082%'
        recjson_value = u'dewey_decimal_classification_number'
        name = u'082x'

    class Tag_167:
        id = 167
        value = u'083%'
        recjson_value = u''
        name = u'083x'

    class Tag_168:
        id = 168
        value = u'084%'
        recjson_value = u'other_report_number'
        name = u'084x'

    class Tag_169:
        id = 169
        value = u'085%'
        recjson_value = u''
        name = u'085x'

    class Tag_170:
        id = 170
        value = u'086%'
        recjson_value = u''
        name = u'086x'

    class Tag_171:
        id = 171
        value = u'240%'
        recjson_value = u''
        name = u'240x'

    class Tag_172:
        id = 172
        value = u'242%'
        recjson_value = u'title_translation'
        name = u'242x'

    class Tag_173:
        id = 173
        value = u'243%'
        recjson_value = u''
        name = u'243x'

    class Tag_174:
        id = 174
        value = u'244%'
        recjson_value = u''
        name = u'244x'

    class Tag_175:
        id = 175
        value = u'247%'
        recjson_value = u''
        name = u'247x'

    class Tag_176:
        id = 176
        value = u'521%'
        recjson_value = u''
        name = u'521x'

    class Tag_177:
        id = 177
        value = u'522%'
        recjson_value = u''
        name = u'522x'

    class Tag_178:
        id = 178
        value = u'524%'
        recjson_value = u''
        name = u'524x'

    class Tag_179:
        id = 179
        value = u'525%'
        recjson_value = u''
        name = u'525x'

    class Tag_180:
        id = 180
        value = u'526%'
        recjson_value = u''
        name = u'526x'

    class Tag_181:
        id = 181
        value = u'650%'
        recjson_value = u'subject'
        name = u'650x'

    class Tag_182:
        id = 182
        value = u'651%'
        recjson_value = u''
        name = u'651x'

    class Tag_183:
        id = 183
        value = u'6531_v'
        recjson_value = u'keywords.v'
        name = u'6531_v'

    class Tag_184:
        id = 184
        value = u'6531_y'
        recjson_value = u'keywords.y'
        name = u'6531_y'

    class Tag_185:
        id = 185
        value = u'6531_9'
        recjson_value = u'keywords.institute'
        name = u'6531_9'

    class Tag_186:
        id = 186
        value = u'654%'
        recjson_value = u''
        name = u'654x'

    class Tag_187:
        id = 187
        value = u'655%'
        recjson_value = u''
        name = u'655x'

    class Tag_188:
        id = 188
        value = u'656%'
        recjson_value = u''
        name = u'656x'

    class Tag_189:
        id = 189
        value = u'657%'
        recjson_value = u''
        name = u'657x'

    class Tag_190:
        id = 190
        value = u'658%'
        recjson_value = u''
        name = u'658x'

    class Tag_191:
        id = 191
        value = u'711%'
        recjson_value = u''
        name = u'711x'

    class Tag_192:
        id = 192
        value = u'900%'
        recjson_value = u'meeting_name'
        name = u'900x'

    class Tag_193:
        id = 193
        value = u'901%'
        recjson_value = u'affiliation'
        name = u'901x'

    class Tag_194:
        id = 194
        value = u'902%'
        recjson_value = u''
        name = u'902x'

    class Tag_195:
        id = 195
        value = u'903%'
        recjson_value = u''
        name = u'903x'

    class Tag_196:
        id = 196
        value = u'904%'
        recjson_value = u''
        name = u'904x'

    class Tag_197:
        id = 197
        value = u'905%'
        recjson_value = u''
        name = u'905x'

    class Tag_198:
        id = 198
        value = u'906%'
        recjson_value = u''
        name = u'906x'

    class Tag_199:
        id = 199
        value = u'907%'
        recjson_value = u''
        name = u'907x'

    class Tag_200:
        id = 200
        value = u'908%'
        recjson_value = u''
        name = u'908x'

    class Tag_201:
        id = 201
        value = u'909C1%'
        recjson_value = u'FIXME_project_info'
        name = u'909C1x'

    class Tag_202:
        id = 202
        value = u'909C5%'
        recjson_value = u'FIXME_909C5'
        name = u'909C5x'

    class Tag_203:
        id = 203
        value = u'909CS%'
        recjson_value = u'FIXME_909CS'
        name = u'909CSx'

    class Tag_204:
        id = 204
        value = u'909CO%'
        recjson_value = u'FIXME_OAI'
        name = u'909COx'

    class Tag_205:
        id = 205
        value = u'909CK%'
        recjson_value = u'FIXME_publishedin'
        name = u'909CKx'

    class Tag_206:
        id = 206
        value = u'909CP%'
        recjson_value = u'photo_information'
        name = u'909CPx'

    class Tag_207:
        id = 207
        value = u'981%'
        recjson_value = u''
        name = u'981x'

    class Tag_208:
        id = 208
        value = u'982%'
        recjson_value = u''
        name = u'982x'

    class Tag_209:
        id = 209
        value = u'983%'
        recjson_value = u''
        name = u'983x'

    class Tag_210:
        id = 210
        value = u'984%'
        recjson_value = u''
        name = u'984x'

    class Tag_211:
        id = 211
        value = u'985%'
        recjson_value = u''
        name = u'985x'

    class Tag_212:
        id = 212
        value = u'986%'
        recjson_value = u''
        name = u'986x'

    class Tag_213:
        id = 213
        value = u'987%'
        recjson_value = u''
        name = u'987x'

    class Tag_214:
        id = 214
        value = u'988%'
        recjson_value = u''
        name = u'988x'

    class Tag_215:
        id = 215
        value = u'989%'
        recjson_value = u''
        name = u'989x'

    class Tag_216:
        id = 216
        value = u'100__0'
        recjson_value = u''
        name = u'author control'

    class Tag_217:
        id = 217
        value = u'110__0'
        recjson_value = u''
        name = u'institute control'

    class Tag_218:
        id = 218
        value = u'130__0'
        recjson_value = u''
        name = u'journal control'

    class Tag_219:
        id = 219
        value = u'150__0'
        recjson_value = u''
        name = u'subject control'

    class Tag_220:
        id = 220
        value = u'260__0'
        recjson_value = u''
        name = u'additional institute control'

    class Tag_221:
        id = 221
        value = u'700__0'
        recjson_value = u''
        name = u'additional author control'

    class Tag_222:
        id = 222
        value = u'003'
        recjson_value = u'agency_code'
        name = u'agency code'

    class Tag_223:
        id = 223
        value = u'909C0b'
        recjson_value = u'FIXME_ALEPH_base_number'
        name = u'FIXME_ALEPH_base_number'

    class Tag_224:
        id = 224
        value = u'909C0a'
        recjson_value = u'FIXME_accelerator'
        name = u'FIXME_accelerator'

    class Tag_225:
        id = 225
        value = u'909C0o'
        recjson_value = u'FIXME_code'
        name = u'FIXME_code'

    class Tag_226:
        id = 226
        value = u'909C2%'
        recjson_value = u'FIXME_909C2'
        name = u'FIXME_909C2'


class FieldTagData(DataSet):

    class FieldTag_10_11:
        score = 100
        id_tag = TagData.Tag_11.ref('id')
        id_field = FieldData.Field_10.ref('id')

    class FieldTag_11_14:
        score = 100
        id_tag = TagData.Tag_14.ref('id')
        id_field = FieldData.Field_11.ref('id')

    class FieldTag_12_15:
        score = 10
        id_tag = TagData.Tag_15.ref('id')
        id_field = FieldData.Field_12.ref('id')

    class FieldTag_13_116:
        score = 10
        id_tag = TagData.Tag_116.ref('id')
        id_field = FieldData.Field_13.ref('id')

    class FieldTag_14_117:
        score = 100
        id_tag = TagData.Tag_117.ref('id')
        id_field = FieldData.Field_14.ref('id')

    class FieldTag_15_118:
        score = 100
        id_tag = TagData.Tag_118.ref('id')
        id_field = FieldData.Field_15.ref('id')

    class FieldTag_16_119:
        score = 100
        id_tag = TagData.Tag_119.ref('id')
        id_field = FieldData.Field_16.ref('id')

    class FieldTag_17_120:
        score = 100
        id_tag = TagData.Tag_120.ref('id')
        id_field = FieldData.Field_17.ref('id')

    #class FieldTag_18_120:
    #    score = 100
    #    id_tag = TagData.Tag_121.ref('id')
    #    id_field = FieldData.Field_18.ref('id')

    class FieldTag_19_131:
        score = 100
        id_tag = TagData.Tag_131.ref('id')
        id_field = FieldData.Field_19.ref('id')

    class FieldTag_20_132:
        score = 100
        id_tag = TagData.Tag_132.ref('id')
        id_field = FieldData.Field_20.ref('id')

    class FieldTag_21_133:
        score = 100
        id_tag = TagData.Tag_133.ref('id')
        id_field = FieldData.Field_21.ref('id')

    class FieldTag_21_134:
        score = 90
        id_tag = TagData.Tag_134.ref('id')
        id_field = FieldData.Field_21.ref('id')

    class FieldTag_22_1:
        score = 100
        id_tag = TagData.Tag_1.ref('id')
        id_field = FieldData.Field_22.ref('id')

    class FieldTag_22_2:
        score = 90
        id_tag = TagData.Tag_2.ref('id')
        id_field = FieldData.Field_22.ref('id')

    class FieldTag_27_135:
        score = 100
        id_tag = TagData.Tag_135.ref('id')
        id_field = FieldData.Field_27.ref('id')

    class FieldTag_28_1:
        score = 100
        id_tag = TagData.Tag_1.ref('id')
        id_field = FieldData.Field_28.ref('id')

    class FieldTag_29_1:
        score = 100
        id_tag = TagData.Tag_1.ref('id')
        id_field = FieldData.Field_29.ref('id')

    class FieldTag_2_3:
        score = 100
        id_tag = TagData.Tag_3.ref('id')
        id_field = FieldData.Field_2.ref('id')

    class FieldTag_2_4:
        score = 90
        id_tag = TagData.Tag_4.ref('id')
        id_field = FieldData.Field_2.ref('id')

    class FieldTag_30_1:
        score = 100
        id_tag = TagData.Tag_1.ref('id')
        id_field = FieldData.Field_30.ref('id')

    class FieldTag_30_2:
        score = 90
        id_tag = TagData.Tag_2.ref('id')
        id_field = FieldData.Field_30.ref('id')

    class FieldTag_32_3:
        score = 100
        id_tag = TagData.Tag_3.ref('id')
        id_field = FieldData.Field_32.ref('id')

    class FieldTag_32_4:
        score = 90
        id_tag = TagData.Tag_4.ref('id')
        id_field = FieldData.Field_32.ref('id')

    class FieldTag_3_1:
        score = 100
        id_tag = TagData.Tag_1.ref('id')
        id_field = FieldData.Field_3.ref('id')

    class FieldTag_3_2:
        score = 90
        id_tag = TagData.Tag_2.ref('id')
        id_field = FieldData.Field_3.ref('id')

    class FieldTag_4_5:
        score = 100
        id_tag = TagData.Tag_5.ref('id')
        id_field = FieldData.Field_4.ref('id')

    class FieldTag_5_6:
        score = 100
        id_tag = TagData.Tag_6.ref('id')
        id_field = FieldData.Field_5.ref('id')

    class FieldTag_6_7:
        score = 30
        id_tag = TagData.Tag_7.ref('id')
        id_field = FieldData.Field_6.ref('id')

    class FieldTag_6_8:
        score = 10
        id_tag = TagData.Tag_8.ref('id')
        id_field = FieldData.Field_6.ref('id')

    class FieldTag_6_9:
        score = 20
        id_tag = TagData.Tag_9.ref('id')
        id_field = FieldData.Field_6.ref('id')

    class FieldTag_7_12:
        score = 100
        id_tag = TagData.Tag_12.ref('id')
        id_field = FieldData.Field_7.ref('id')

    class FieldTag_7_13:
        score = 90
        id_tag = TagData.Tag_13.ref('id')
        id_field = FieldData.Field_7.ref('id')

    class FieldTag_8_10:
        score = 100
        id_tag = TagData.Tag_10.ref('id')
        id_field = FieldData.Field_8.ref('id')

    class FieldTag_9_115:
        score = 100
        id_tag = TagData.Tag_115.ref('id')
        id_field = FieldData.Field_9.ref('id')

    class FieldTag_33_1:
        score = 100
        id_tag = TagData.Tag_1.ref('id')
        id_field = FieldData.Field_33.ref('id')

    class FieldTag_33_146:
        score = 100
        id_tag = TagData.Tag_146.ref('id')
        id_field = FieldData.Field_33.ref('id')

    class FieldTag_33_140:
        score = 100
        id_tag = TagData.Tag_140.ref('id')
        id_field = FieldData.Field_33.ref('id')

    class FieldTag_34_148:
        score = 100
        id_tag = TagData.Tag_148.ref('id')
        id_field = FieldData.Field_34.ref('id')

    class FieldTag_34_149:
        score = 100
        id_tag = TagData.Tag_149.ref('id')
        id_field = FieldData.Field_34.ref('id')

    class FieldTag_34_150:
        score = 100
        id_tag = TagData.Tag_150.ref('id')
        id_field = FieldData.Field_34.ref('id')

    class FieldTag_35_151:
        score = 100
        id_tag = TagData.Tag_151.ref('id')
        id_field = FieldData.Field_35.ref('id')

    class FieldTag_35_152:
        score = 100
        id_tag = TagData.Tag_152.ref('id')
        id_field = FieldData.Field_35.ref('id')

    class FieldTag_35_153:
        score = 100
        id_tag = TagData.Tag_153.ref('id')
        id_field = FieldData.Field_35.ref('id')

    class FieldTag_36_154:
        score = 100
        id_tag = TagData.Tag_154.ref('id')
        id_field = FieldData.Field_36.ref('id')

    class FieldTag_36_155:
        score = 100
        id_tag = TagData.Tag_155.ref('id')
        id_field = FieldData.Field_36.ref('id')

    class FieldTag_36_156:
        score = 100
        id_tag = TagData.Tag_156.ref('id')
        id_field = FieldData.Field_36.ref('id')

    class FieldTag_39_17:
        score = 10
        id_tag = TagData.Tag_17.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_18:
        score = 10
        id_tag = TagData.Tag_18.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_157:
        score = 10
        id_tag = TagData.Tag_157.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_158:
        score = 10
        id_tag = TagData.Tag_158.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_159:
        score = 10
        id_tag = TagData.Tag_159.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_160:
        score = 10
        id_tag = TagData.Tag_160.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_161:
        score = 10
        id_tag = TagData.Tag_161.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_162:
        score = 10
        id_tag = TagData.Tag_162.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_163:
        score = 10
        id_tag = TagData.Tag_163.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_164:
        score = 10
        id_tag = TagData.Tag_164.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_20:
        score = 10
        id_tag = TagData.Tag_20.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_21:
        score = 10
        id_tag = TagData.Tag_21.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_22:
        score = 10
        id_tag = TagData.Tag_22.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_23:
        score = 10
        id_tag = TagData.Tag_23.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_165:
        score = 10
        id_tag = TagData.Tag_165.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_166:
        score = 10
        id_tag = TagData.Tag_166.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_167:
        score = 10
        id_tag = TagData.Tag_167.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_168:
        score = 10
        id_tag = TagData.Tag_168.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_169:
        score = 10
        id_tag = TagData.Tag_169.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_170:
        score = 10
        id_tag = TagData.Tag_170.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_25:
        score = 10
        id_tag = TagData.Tag_25.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_27:
        score = 10
        id_tag = TagData.Tag_27.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_28:
        score = 10
        id_tag = TagData.Tag_28.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_29:
        score = 10
        id_tag = TagData.Tag_29.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_30:
        score = 10
        id_tag = TagData.Tag_30.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_31:
        score = 10
        id_tag = TagData.Tag_31.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_32:
        score = 10
        id_tag = TagData.Tag_32.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_33:
        score = 10
        id_tag = TagData.Tag_33.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_34:
        score = 10
        id_tag = TagData.Tag_34.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_35:
        score = 10
        id_tag = TagData.Tag_35.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_36:
        score = 10
        id_tag = TagData.Tag_36.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_37:
        score = 10
        id_tag = TagData.Tag_37.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_38:
        score = 10
        id_tag = TagData.Tag_38.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_39:
        score = 10
        id_tag = TagData.Tag_39.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_171:
        score = 10
        id_tag = TagData.Tag_171.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_172:
        score = 10
        id_tag = TagData.Tag_172.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_173:
        score = 10
        id_tag = TagData.Tag_173.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_174:
        score = 10
        id_tag = TagData.Tag_174.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_175:
        score = 10
        id_tag = TagData.Tag_175.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_41:
        score = 10
        id_tag = TagData.Tag_41.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_42:
        score = 10
        id_tag = TagData.Tag_42.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_43:
        score = 10
        id_tag = TagData.Tag_43.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_44:
        score = 10
        id_tag = TagData.Tag_44.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_45:
        score = 10
        id_tag = TagData.Tag_45.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_46:
        score = 10
        id_tag = TagData.Tag_46.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_47:
        score = 10
        id_tag = TagData.Tag_47.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_48:
        score = 10
        id_tag = TagData.Tag_48.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_49:
        score = 10
        id_tag = TagData.Tag_49.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_50:
        score = 10
        id_tag = TagData.Tag_50.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_51:
        score = 10
        id_tag = TagData.Tag_51.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_52:
        score = 10
        id_tag = TagData.Tag_52.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_53:
        score = 10
        id_tag = TagData.Tag_53.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_54:
        score = 10
        id_tag = TagData.Tag_54.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_55:
        score = 10
        id_tag = TagData.Tag_55.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_56:
        score = 10
        id_tag = TagData.Tag_56.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_57:
        score = 10
        id_tag = TagData.Tag_57.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_58:
        score = 10
        id_tag = TagData.Tag_58.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_59:
        score = 10
        id_tag = TagData.Tag_59.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_60:
        score = 10
        id_tag = TagData.Tag_60.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_61:
        score = 10
        id_tag = TagData.Tag_61.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_62:
        score = 10
        id_tag = TagData.Tag_62.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_63:
        score = 10
        id_tag = TagData.Tag_63.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_64:
        score = 10
        id_tag = TagData.Tag_64.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_65:
        score = 10
        id_tag = TagData.Tag_65.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_66:
        score = 10
        id_tag = TagData.Tag_66.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_67:
        score = 10
        id_tag = TagData.Tag_67.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_176:
        score = 10
        id_tag = TagData.Tag_176.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_177:
        score = 10
        id_tag = TagData.Tag_177.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_178:
        score = 10
        id_tag = TagData.Tag_178.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_179:
        score = 10
        id_tag = TagData.Tag_179.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_180:
        score = 10
        id_tag = TagData.Tag_180.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_69:
        score = 10
        id_tag = TagData.Tag_69.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_70:
        score = 10
        id_tag = TagData.Tag_70.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_71:
        score = 10
        id_tag = TagData.Tag_71.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_72:
        score = 10
        id_tag = TagData.Tag_72.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_73:
        score = 10
        id_tag = TagData.Tag_73.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_74:
        score = 10
        id_tag = TagData.Tag_74.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_75:
        score = 10
        id_tag = TagData.Tag_75.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_76:
        score = 10
        id_tag = TagData.Tag_76.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_77:
        score = 10
        id_tag = TagData.Tag_77.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_78:
        score = 10
        id_tag = TagData.Tag_78.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_79:
        score = 10
        id_tag = TagData.Tag_79.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_80:
        score = 10
        id_tag = TagData.Tag_80.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_181:
        score = 10
        id_tag = TagData.Tag_181.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_182:
        score = 10
        id_tag = TagData.Tag_182.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_183:
        score = 10
        id_tag = TagData.Tag_183.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_184:
        score = 10
        id_tag = TagData.Tag_184.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_185:
        score = 10
        id_tag = TagData.Tag_185.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_186:
        score = 10
        id_tag = TagData.Tag_186.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_82:
        score = 10
        id_tag = TagData.Tag_82.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_83:
        score = 10
        id_tag = TagData.Tag_83.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_84:
        score = 10
        id_tag = TagData.Tag_84.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_85:
        score = 10
        id_tag = TagData.Tag_85.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_187:
        score = 10
        id_tag = TagData.Tag_187.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_88:
        score = 10
        id_tag = TagData.Tag_88.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_89:
        score = 10
        id_tag = TagData.Tag_89.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_90:
        score = 10
        id_tag = TagData.Tag_90.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_91:
        score = 10
        id_tag = TagData.Tag_91.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_92:
        score = 10
        id_tag = TagData.Tag_92.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_93:
        score = 10
        id_tag = TagData.Tag_93.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_94:
        score = 10
        id_tag = TagData.Tag_94.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_95:
        score = 10
        id_tag = TagData.Tag_95.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_96:
        score = 10
        id_tag = TagData.Tag_96.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_97:
        score = 10
        id_tag = TagData.Tag_97.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_98:
        score = 10
        id_tag = TagData.Tag_98.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_99:
        score = 10
        id_tag = TagData.Tag_99.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_100:
        score = 10
        id_tag = TagData.Tag_100.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_102:
        score = 10
        id_tag = TagData.Tag_102.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_103:
        score = 10
        id_tag = TagData.Tag_103.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_104:
        score = 10
        id_tag = TagData.Tag_104.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_105:
        score = 10
        id_tag = TagData.Tag_105.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_188:
        score = 10
        id_tag = TagData.Tag_188.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_189:
        score = 10
        id_tag = TagData.Tag_189.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_190:
        score = 10
        id_tag = TagData.Tag_190.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_191:
        score = 10
        id_tag = TagData.Tag_191.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_192:
        score = 10
        id_tag = TagData.Tag_192.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_193:
        score = 10
        id_tag = TagData.Tag_193.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_194:
        score = 10
        id_tag = TagData.Tag_194.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_195:
        score = 10
        id_tag = TagData.Tag_195.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_196:
        score = 10
        id_tag = TagData.Tag_196.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_107:
        score = 10
        id_tag = TagData.Tag_107.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_108:
        score = 10
        id_tag = TagData.Tag_108.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_109:
        score = 10
        id_tag = TagData.Tag_109.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_110:
        score = 10
        id_tag = TagData.Tag_110.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_111:
        score = 10
        id_tag = TagData.Tag_111.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_112:
        score = 10
        id_tag = TagData.Tag_112.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_113:
        score = 10
        id_tag = TagData.Tag_113.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_197:
        score = 10
        id_tag = TagData.Tag_197.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_198:
        score = 10
        id_tag = TagData.Tag_198.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_199:
        score = 10
        id_tag = TagData.Tag_199.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_200:
        score = 10
        id_tag = TagData.Tag_200.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_201:
        score = 10
        id_tag = TagData.Tag_201.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_202:
        score = 10
        id_tag = TagData.Tag_202.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_203:
        score = 10
        id_tag = TagData.Tag_203.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_204:
        score = 10
        id_tag = TagData.Tag_204.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_205:
        score = 10
        id_tag = TagData.Tag_205.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_206:
        score = 10
        id_tag = TagData.Tag_206.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_207:
        score = 10
        id_tag = TagData.Tag_207.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_208:
        score = 10
        id_tag = TagData.Tag_208.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_209:
        score = 10
        id_tag = TagData.Tag_209.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_210:
        score = 10
        id_tag = TagData.Tag_210.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_211:
        score = 10
        id_tag = TagData.Tag_211.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_212:
        score = 10
        id_tag = TagData.Tag_212.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_213:
        score = 10
        id_tag = TagData.Tag_213.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_214:
        score = 10
        id_tag = TagData.Tag_214.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_215:
        score = 10
        id_tag = TagData.Tag_215.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_122:
        score = 10
        id_tag = TagData.Tag_122.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_123:
        score = 10
        id_tag = TagData.Tag_123.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_124:
        score = 10
        id_tag = TagData.Tag_124.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_125:
        score = 10
        id_tag = TagData.Tag_125.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_126:
        score = 10
        id_tag = TagData.Tag_126.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_127:
        score = 10
        id_tag = TagData.Tag_127.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_128:
        score = 10
        id_tag = TagData.Tag_128.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_129:
        score = 10
        id_tag = TagData.Tag_129.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_130:
        score = 10
        id_tag = TagData.Tag_130.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_1:
        score = 10
        id_tag = TagData.Tag_1.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_2:
        score = 10
        id_tag = TagData.Tag_2.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_216:
        score = 10
        id_tag = TagData.Tag_216.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_217:
        score = 10
        id_tag = TagData.Tag_217.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_218:
        score = 10
        id_tag = TagData.Tag_218.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_219:
        score = 10
        id_tag = TagData.Tag_219.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_220:
        score = 10
        id_tag = TagData.Tag_220.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_221:
        score = 10
        id_tag = TagData.Tag_221.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_223:
        score = 10
        id_tag = TagData.Tag_223.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_224:
        score = 10
        id_tag = TagData.Tag_224.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_225:
        score = 10
        id_tag = TagData.Tag_225.ref('id')
        id_field = FieldData.Field_39.ref('id')

    class FieldTag_39_226:
        score = 10
        id_tag = TagData.Tag_226.ref('id')
        id_field = FieldData.Field_39.ref('id')



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 60 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""Aggregation parser extension."""

from pyparsing import Suppress, OneOrMore, Word, alphanums, nestedExpr, \
    originalTextFor, Keyword

from invenio.modules.jsonalchemy.parser import FieldBaseExtensionParser, \
    FieldParser, PYTHON_ALLOWED_EXPR, indentedBlock


class AggregationParser(FieldBaseExtensionParser):

    """Handles the aggregation section from a field definition.

    An example of this section could be::

        title:
            aggregation:
                title(), {'<aggregation_type>': '<aggregagtion_body>'}

        authors:
            aggregation:
                author('native'), {....}
                author('elasticsearch'), {....}

    The parameter passed to the aggregation could be used by the aggregation
    for example to decide if the current aggregation rule will be applied
    depending on the search engine.
    """

    __parsername__ = 'aggregation'

    @classmethod
    def parse_element(cls, indent_stack):
        """Set to the rule the list of aggregations."""
        aggregation_body = (
            Word(alphanums + "_") +
            originalTextFor(nestedExpr()) +
            Suppress(',') +
            PYTHON_ALLOWED_EXPR
        ).setParseAction(lambda toks: {
            'name': toks[0],
            'engine': eval(toks[1]),
            'rule': eval(toks[2])
        })
        return (Keyword('aggregation:').suppress() +
                indentedBlock(OneOrMore(aggregation_body), indent_stack)
                ).setResultsName('aggregation')

    @classmethod
    def create_element(cls, rule, namespace):
        """Prepare the list of aggregations with their names and parameters."""
        id_ = rule.field['json_id']

        aggregations = {} if not rule.extend else \
            FieldParser.field_definitions(namespace)[id_].get(
                'aggregation', {})
        for aggregation in rule.aggregation:
            if aggregation['name'] not in aggregations:
                aggregations[aggregation['name']] = []
            aggregations[aggregation['name']].append((
                aggregation['engine'], aggregation['rule']))
        return aggregations

parser = AggregationParser



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2012 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
"""
WebSearch service to display LHC beam status
"""
from invenio_search.services import SearchService
from invenio.base.i18n import gettext_set_language
from invenio.config import CFG_SITE_LANG, CFG_SITE_URL

__plugin_version__ = "Search Service Plugin API 1.0"

class LHCBeamStatusService(SearchService):
    """
    Display LHC Beam Status
    """

    def get_description(self, ln=CFG_SITE_LANG):
        "Return service description"
        return "Return LHC Beam status info"

    def answer(self, req, user_info, of, cc, colls_to_search, p, f, search_units, ln):
        """
        Answer question given by context.

        Return (relevance, html_string) where relevance is integer
        from 0 to 100 indicating how relevant to the question the
        answer is (see C{CFG_WEBSEARCH_SERVICE_MAX_SERVICE_ANSWER_RELEVANCE} for details) ,
        and html_string being a formatted answer.
        """
        if f:
            return (0, '')

        words = [unit[1].lower() for unit in search_units if unit[2] == ""]

        if not words:
            return (0, '')

        _ = gettext_set_language(ln)
        if 'vistars' in words or \
           (('lhc' in words or 'beam' in words) and \
            'status' in words):
            out = '''
            <img id="vistar" src="%(CFG_SITE_URL)s/img/loading.gif"/>
<script language="javascript" type="text/javascript">
function refresh()
{
    imgobj = $("#vistar")
	imgobj.attr("src", 'http://cs-ccr-www3.cern.ch/vistar_capture/lhc1.png'+ '?'+Math.random()).stop(true,true).hide().show();
    imgobj.attr("style", "max-width:600px");
	setTimeout("refresh()", 8000);
}
$(document).ready(function(){
	refresh();
});
</script>
'''  % {'CFG_SITE_URL': CFG_SITE_URL}

            return (70, out)

        return (0, '')



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""Query results cacher."""

from invenio_search.cache import get_results_cache, set_results_cache


class CacheOp(object):

    """Store results in cache."""

    def __init__(self, query, collection=None):
        """Define query that should be cached."""
        self.query = query
        self.collection = collection

    def __repr__(self):
        """Object representation."""
        return "%s(%s)" % (self.__class__.__name__, repr(self.query))

    def accept(self, visitor):
        """Store intermediate results to the cache."""
        results = get_results_cache(str(self.query), self.collection)
        if results is None:
            results = self.query.accept(visitor)
            set_results_cache(results, str(self.query), self.collection)
        return results


def apply(query, collection=None, **kwargs):
    """Decorate query with a cache operator."""
    return CacheOp(query, collection=collection)
